package cz.muni.fi.dao;

import cz.muni.fi.domain.DisagreedFeatureRate;
import cz.muni.fi.domain.Feature;
import cz.muni.fi.domain.Participant;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;
import java.util.List;

/**
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 1.3.12
 */
@Repository("disagreedFeatureRateDao")
public class JpaDisagreedFeatureRateDao implements DisagreedFeatureRateDao {

    private EntityManager entityManager;

    @PersistenceContext
    public void setEntityManager(EntityManager entityManager) {
        this.entityManager = entityManager;
    }

    @Override
    @Transactional
    public DisagreedFeatureRate findById(Long id) {
        return entityManager.find(DisagreedFeatureRate.class, id);
    }

    @Override
    @Transactional
    public void save(DisagreedFeatureRate disagreedFeatureRate) {
        entityManager.persist(disagreedFeatureRate);
    }

    @Override
    @Transactional
    public DisagreedFeatureRate update(DisagreedFeatureRate disagreedFeatureRate) {
        return entityManager.merge(disagreedFeatureRate);
    }

    @Override
    @Transactional
    public void delete(DisagreedFeatureRate disagreedFeatureRate) {
        DisagreedFeatureRate feat1 = entityManager.find(DisagreedFeatureRate.class, disagreedFeatureRate.getId());
        entityManager.remove(feat1);
    }

    @Override
    @Transactional
    public List<DisagreedFeatureRate> getAllDisagreedFeatureRates(Feature feature) {
        Query query = entityManager.createQuery("select s from DisagreedFeatureRate s where s.feature = :feature order by s.id");
        query.setParameter("feature", feature);
        return query.getResultList();
    }

    @Override
    @Transactional
    public List<DisagreedFeatureRate> getAllDisagreedFeatureRates(Participant participant) {
        Query query = entityManager.createQuery("select s from DisagreedFeatureRate s where s.student = :participant or s.manager = :participant order by s.id");
        query.setParameter("participant", participant);
        return query.getResultList();
    }
}
