package cz.muni.fi.dao;

import cz.muni.fi.domain.Feature;
import cz.muni.fi.domain.Period;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;
import java.util.ArrayList;
import java.util.List;

/**
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 9.8.11
 */
@Repository("featureDao")
public class JpaFeatureDao implements FeatureDao {

    private EntityManager entityManager;

    @PersistenceContext
    public void setEntityManager(EntityManager entityManager) {
        this.entityManager = entityManager;
    }

    @Transactional
    @Override
    public Feature findById(Long id) {
        Feature result = entityManager.find(Feature.class, id);
        if (result != null) {
            if (result.getSubFeatures() == null) {
                result.setSubFeatures(new ArrayList<Feature>());
            }
        }
        return result;
    }

    @Transactional
    @Override
    public void save(Feature feature) {
        entityManager.persist(feature);
    }

    @Override
    @Transactional
    public Feature update(Feature feature) {
        if (feature.getParent() != null) {
            entityManager.merge(feature.getParent());
        }
        return entityManager.merge(feature);
    }

    @Transactional
    @Override
    public void delete(Feature feature) {
        Feature feat1 = entityManager.find(Feature.class, feature.getId());
        entityManager.remove(feat1);
    }

    @Transactional
    @Override
    public List<Feature> getAllFeatures() {
        return entityManager.createQuery("select s from Feature s order by s.id").getResultList();
    }

    @Override
    @Transactional
    public List<Feature> getAllRootFetures() {
        return entityManager.createQuery("select s from Feature s where s.parent IS  NULL order by s.id").getResultList();
    }

    @Override
    @Transactional
    public List<Feature> getAllFeaturesWithoutRateInPeriod(Period period) {
        Query query = entityManager.createQuery("select f from Feature f where f not in" +
                "(select fr.feature from FeatureRate fr where fr.period = :period) order by f.id");
        query.setParameter("period", period);

        return query.getResultList();
    }
}
