package cz.muni.fi.dao;

import cz.muni.fi.domain.Feature;
import cz.muni.fi.domain.FeatureRate;
import cz.muni.fi.domain.Organization;
import cz.muni.fi.domain.Period;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;
import java.util.List;

/**
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 15.8.11
 */
@Repository("featureRateDao")
public class JpaFeatureRateDao implements FeatureRateDao {
    private EntityManager entityManager;

    @PersistenceContext
    public void setEntityManager(EntityManager entityManager) {
        this.entityManager = entityManager;
    }

    @Override
    @Transactional
    public FeatureRate findById(Long id) {
        return entityManager.find(FeatureRate.class, id);
    }

    @Override
    @Transactional
    public void save(FeatureRate featureRate) {
        entityManager.persist(featureRate);
    }

    @Override
    @Transactional
    public FeatureRate update(FeatureRate featureRate) {
        return entityManager.merge(featureRate);
    }

    @Override
    @Transactional
    public void delete(FeatureRate featureRate) {
        FeatureRate fr = entityManager.find(FeatureRate.class, featureRate.getId());
        entityManager.remove(fr);
    }

    @Override
    @Transactional
    public List<FeatureRate> getAllFeatureRates(Period period) {
        Query query = entityManager.createQuery("select s from FeatureRate s where s.period = :period order by s.feature.id");
        query.setParameter("period", period);
        List<FeatureRate> list = query.getResultList();
        return list;
    }

    @Override
    @Transactional
    public List<FeatureRate> getAllFeatureRates(Organization organization) {
        Query query = entityManager.createQuery("select s from FeatureRate s where s.organization = :organization order by s.feature.id");
        query.setParameter("organization", organization);
        List<FeatureRate> list = query.getResultList();
        return list;
    }

    @Override
    @Transactional
    public List<FeatureRate> findByFeature(Feature feature, Period period) {
        Query query = entityManager.createQuery("select s from FeatureRate s where s.feature = :feature and s.period = :period");
        query.setParameter("feature", feature);
        query.setParameter("period", period);
        List<FeatureRate> list = query.getResultList();
        return list;
    }

    @Override
    @Transactional
    public List<FeatureRate> findByFeature(Feature feature, Organization organization) {
        Query query = entityManager.createQuery("select s from FeatureRate s where s.feature = :feature and s.organization = :organization");
        query.setParameter("feature", feature);
        query.setParameter("organization", organization);
        List<FeatureRate> list = query.getResultList();
        return list;
    }


    @Override
    @Transactional
    public FeatureRate findByFeature(Feature feature, Period period, Organization organization) {
        Query query = entityManager.createQuery("select s from FeatureRate s where s.feature = :feature and s.period = :period and s.organization = :organization");
        query.setParameter("feature", feature);
        query.setParameter("period", period);
        query.setParameter("organization", organization);
        List<FeatureRate> list = query.getResultList();
        if (list.size() != 1) {
            return null;
        } else {
            return list.get(0);
        }
    }

    @Override
    public List<FeatureRate> getAllFeatureRates(Period period, Organization organization) {
        Query query = entityManager.createQuery("select s from FeatureRate s where s.period = :period and s.organization = :organization order by s.feature.id");
        query.setParameter("period", period);
        query.setParameter("organization", organization);
        return query.getResultList();
    }


}
