package cz.muni.fi.dao;

import cz.muni.fi.domain.Form;
import cz.muni.fi.domain.Period;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;
import java.util.List;

/**
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 12.8.11
 */
@Repository("formDao")
public class JpaFormDao implements FormDao {

    private EntityManager entityManager;

    @PersistenceContext
    public void setEntityManager(EntityManager entityManager) {
        this.entityManager = entityManager;
    }

    @Override
    @Transactional
    public Form findById(Long id) {
        return entityManager.find(Form.class, id);
    }

    @Override
    @Transactional
    public void save(Form form) {
        entityManager.persist(form);
    }

    @Override
    @Transactional
    public Form update(Form form) {
        return entityManager.merge(form);
    }

    @Override
    @Transactional
    public void delete(Form form) {
        Form f = entityManager.find(Form.class, form.getId());
        entityManager.remove(f);
    }

    @Override
    @Transactional
    public List<Form> getAllForms() {
        List<Form> forms = entityManager.createQuery("select s from Form s order by s.state,s.timestamp").getResultList();
        for (Form form : forms) {
            form.getFeatureRatingList().size();
        }
        return forms;

    }

    @Override
    @Transactional
    public Form findByHash(String hash) {
        Query query = entityManager.createQuery("select s from Form s where s.hash = :hash");//
        query.setParameter("hash", hash);
        List<Form> list = query.getResultList();
        if (list.size() != 1) {
            return null;
        } else {
            list.get(0).getFeatureRatingList().size();
            return list.get(0);
        }
    }

    @Override
    @Transactional
    public List<Form> getAllForms(Period period) {
        Query query = entityManager.createQuery("select s from Form s where s.period = :period order by s.state, s.timestamp");
        query.setParameter("period", period);
        return query.getResultList();
    }
}
