package cz.muni.fi.dao;

import cz.muni.fi.domain.Form;
import cz.muni.fi.domain.FormPair;
import cz.muni.fi.domain.Organization;
import cz.muni.fi.domain.Period;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;
import java.util.List;

/**
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 12.8.11
 */
@Repository("formPairDao")
public class JpaFormPairDao implements FormPairDao {

    private EntityManager entityManager;

    @PersistenceContext
    public void setEntityManager(EntityManager entityManager) {
        this.entityManager = entityManager;
    }

    @Override
    @Transactional
    public FormPair findById(Long id) {
        FormPair pair = entityManager.find(FormPair.class, id);
        if (pair == null) {
            return null;
        }
        pair.getManagersForm().getFeatureRatingList().size();
        pair.getStudentsForm().getFeatureRatingList().size();
        return pair;
    }

    @Override
    @Transactional
    public void save(FormPair formPair) {
        entityManager.persist(formPair);
    }

    @Override
    @Transactional
    public FormPair update(FormPair formPair) {
        return entityManager.merge(formPair);
    }

    @Override
    @Transactional
    public void delete(FormPair formPair) {
        FormPair fp = entityManager.find(FormPair.class, formPair.getId());
        entityManager.remove(fp);
    }

    @Override
    @Transactional
    public List<FormPair> getAllFormPairs() {
        Query query = entityManager.createQuery("select s from FormPair s order by s.id");
        List<FormPair> result = query.getResultList();
        for (FormPair pair : result) {
            pair.getManagersForm().getFeatureRatingList().size();
            pair.getStudentsForm().getFeatureRatingList().size();
        }
        return result;
    }

    @Override
    @Transactional
    public FormPair findByForm(Form form) {
        Query query = entityManager.createQuery("select s from FormPair s where s.studentsForm = :form or s.managersForm=:form");//
        query.setParameter("form", form);
        List<FormPair> list = query.getResultList();
        if (list.size() != 1) {
            return null;
        } else {
            list.get(0).getManagersForm().getFeatureRatingList().size();
            list.get(0).getStudentsForm().getFeatureRatingList().size();
            return list.get(0);
        }
    }

    @Override
    @Transactional
    public List<FormPair> getAllFormPairs(Period period) {
        Query query = entityManager.createQuery("select s from FormPair s where s.studentsForm.period = :period order by s.id");
        query.setParameter("period", period);
        List<FormPair> result = query.getResultList();
        for (FormPair pair : result) {
            pair.getManagersForm().getFeatureRatingList().size();
            pair.getStudentsForm().getFeatureRatingList().size();
        }
        return result;
    }

    @Override
    @Transactional
    public List<FormPair> getAllFormPairs(Period period, Organization organization) {
        Query query = entityManager.createQuery("select s from FormPair s where s.studentsForm.period = :period and s.managersForm.owner.organization = :org order by s.id");
        query.setParameter("period", period);
        query.setParameter("org", organization);
        List<FormPair> result = query.getResultList();
        for (FormPair pair : result) {
            pair.getManagersForm().getFeatureRatingList().size();
            pair.getStudentsForm().getFeatureRatingList().size();
        }
        return result;
    }

    @Override
    @Transactional
    public List<FormPair> getAllFormPairs(Organization organization) {
        Query query = entityManager.createQuery("select s from FormPair s where s.managersForm.owner.organization = :org order by s.id");
        query.setParameter("org", organization);
        List<FormPair> result = query.getResultList();
        for (FormPair pair : result) {
            pair.getManagersForm().getFeatureRatingList().size();
            pair.getStudentsForm().getFeatureRatingList().size();
        }
        return result;
    }
}
