package cz.muni.fi.dao;

import cz.muni.fi.domain.Organization;
import cz.muni.fi.domain.Participant;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;
import java.util.List;


/**
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 12.8.11
 */

@Repository("organizationDao")
public class JpaOrganizationDao implements OrganizationDao {

    private EntityManager entityManager;

    @PersistenceContext
    public void setEntityManager(EntityManager entityManager) {
        this.entityManager = entityManager;
    }

    @Override
    @Transactional
    public Organization findById(Long id) {
        return entityManager.find(Organization.class, id);
    }

    @Override
    @Transactional
    public void save(Organization organization) {
        entityManager.persist(organization);
    }

    @Override
    @Transactional
    public Organization update(Organization organization) {
        return entityManager.merge(organization);
    }

    @Override
    @Transactional
    public void delete(Organization organization) {
        Organization org1 = entityManager.find(Organization.class, organization.getId());
        entityManager.remove(org1);
    }

    @Override
    @Transactional
    public List<Organization> getAllActiveOrganizations() {
        return entityManager.createQuery("select s from Organization s where s.active = true order by s.name").getResultList();
    }

    @Override
    @Transactional
    public List<Organization> getAllOrganizations() {
        return entityManager.createQuery("select s from Organization s order by s.name").getResultList();
    }

    @Override
    @Transactional
    public Organization findByIdNumber(String idNumber) {
        Query query = entityManager.createQuery("select s from Organization s where s.idNumber= :idno ");
        query.setParameter("idno", idNumber);
        List<Organization> result = query.getResultList();
        if (result.isEmpty()) {
            return null;
        } else {
            return result.get(0);
        }
    }
}
