package cz.muni.fi.dao;

import cz.muni.fi.domain.Form;
import cz.muni.fi.domain.Organization;
import cz.muni.fi.domain.Participant;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;
import java.util.List;

/**
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 12.8.11
 */
@Repository("participantDao")
public class JpaParticipantDao implements ParticipantDao {

    private EntityManager entityManager;

    @PersistenceContext
    public void setEntityManager(EntityManager entityManager) {
        this.entityManager = entityManager;
    }

    @Override
    @Transactional
    public Participant findByEmail(String email) {
        Query query = entityManager.createQuery("select s from Participant s where s.mail= :email ");
        query.setParameter("email", email);
        List<Participant> result = query.getResultList();
        if (result.isEmpty()) {
            return null;
        } else {
            return result.get(0);
        }
    }

    @Override
    @Transactional
    public void save(Participant participant) {
        entityManager.persist(participant);
    }

    @Override
    @Transactional
    public Participant update(Participant participant) {
        return entityManager.merge(participant);
    }

    @Override
    @Transactional
    public void delete(Participant participant) {
        Participant part1 = entityManager.find(Participant.class, participant.getId());
        entityManager.remove(part1);
    }

    @Override
    @Transactional
    public List<Participant> getAllStudents() {
        return entityManager.createQuery("select s from Participant s where s.role= 'student' order by s.surname,s.name").getResultList();

    }

    @Override
    @Transactional
    public List<Participant> getAllManagers() {
        return entityManager.createQuery("select s from Participant s where s.role= 'manager' order by s.surname, s.name").getResultList();
    }

    @Override
    @Transactional
    public boolean canBeDeleted(Participant participant) {
        Query query = entityManager.createQuery("select s from Form s where s.owner = :owner");//
        query.setParameter("owner", participant);
        List<Form> list = query.getResultList();
        return list.isEmpty();
    }

    @Override
    @Transactional
    public Participant findByUco(String studentUco) {
        Query query = entityManager.createQuery("select s from Student s where s.uco= :uco ");
        query.setParameter("uco", studentUco);
        List<Participant> result = query.getResultList();
        if (result.isEmpty()) {
            return null;
        } else {
            return result.get(0);
        }
    }

    @Override
    @Transactional
    public Participant findById(Long id) {
        return entityManager.find(Participant.class, id);
    }

    @Override
    @Transactional
    public List<Participant> getAllManagersFor(Organization org) {
        Query query = entityManager.createQuery("select s from Manager s where s.organization= :org ");
        query.setParameter("org", org);
        return query.getResultList();
    }
}

