package cz.muni.fi.dao;

import cz.muni.fi.domain.Period;
import cz.muni.fi.exception.DataIntegrityViolationException;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import java.util.List;

/**
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 21.11.11
 */
@Repository("periodDao")
public class JpaPeriodDao implements PeriodDao {
    private EntityManager entityManager;

    @PersistenceContext
    public void setEntityManager(EntityManager entityManager) {
        this.entityManager = entityManager;
    }

    @Override
    @Transactional
    public List<Period> getAllPeriods() {
        return entityManager.createQuery("select s from Period s order by s.timestamp").getResultList();
    }

    @Override
    @Transactional
    public Period getById(Long id) {
        return entityManager.find(Period.class, id);
    }

    @Override
    @Transactional
    public void save(Period period) {
        entityManager.persist(period);
    }

    @Override
    @Transactional
    public Period update(Period period) {
        return entityManager.merge(period);
    }

    @Override
    @Transactional
    public void delete(Period period) {
        Period p = entityManager.find(Period.class, period.getId());
        entityManager.remove(p);
    }

    @Override
    @Transactional
    public Period getCurrrent() throws DataIntegrityViolationException {
        List<Period> list = entityManager.createQuery("SELECT p FROM Period p WHERE p.timestamp = (SELECT MAX(p.timestamp) FROM p)").getResultList();
        if (list.isEmpty()) {
            throw new DataIntegrityViolationException("no period found");
        }
        return list.get(0);
    }
}
