package cz.muni.fi.dao;

import cz.muni.fi.domain.Authority;
import cz.muni.fi.domain.Period;
import cz.muni.fi.domain.User;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;
import java.util.List;

/**
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 25.10.11
 */
@Repository("userDao")
public class JpaUserDao implements UserDao {

    private EntityManager entityManager;

    @PersistenceContext
    public void setEntityManager(EntityManager entityManager) {
        this.entityManager = entityManager;
    }

    @Override
    @Transactional
    public User findById(String id) {
        User user = entityManager.find(User.class, id);
        if (user != null) {
            user.getAuthorities().size();
        }
        return user;
    }

    @Transactional
    @Override
    public void save(User user) {
        entityManager.persist(user);
    }

    @Override
    @Transactional
    public User update(User user) {
        if (user.getAuthorities() != null) {
            for (Authority a : user.getAuthorities()) {
                if (a.getId() == null) {
                    entityManager.persist(a);
                }

            }
        }
        return entityManager.merge(user);

    }

    @Override
    @Transactional
    public void delete(User user) {
        User us1 = entityManager.find(User.class, user.getUserName());
        entityManager.remove(us1);
    }

    @Override
    @Transactional
    public List<User> getAllUsers() {
        List<User> result = entityManager.createQuery("select s from User s order by s.userName").getResultList();
        for (User u : result) {
            u.getAuthorities().size();
        }
        return result;
    }

    @Override
    @Transactional
    public List<User> getAllUsersWithSelectedPeriod(Period period) {
        Query query = entityManager.createQuery("select s from User s where s.selectedPeriod = :period order by s.userName");
        query.setParameter("period", period);
        List<User> result = query.getResultList();
         for (User u : result) {
            u.getAuthorities().size();
        }
        return result;
    }

}