package cz.muni.fi.jstl;

import cz.muni.fi.domain.Authority;
import org.apache.commons.codec.binary.Base64;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.List;

/**
 * Custom functions used in custom tag library definition.
 * These functions are declared as tld tags in functions.tld.
 *
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 3.11.11
 */
public final class Functions {
    private Functions() {
    }

    /**
     * Performs a Base64 encoding of given string.
     *
     * @param string string to encode
     * @return base64 encode string
     * @throws UnsupportedEncodingException if utf8 is not supported on the platform
     */
    public static String urlEncode(String string) throws UnsupportedEncodingException {
        return new String(Base64.encodeBase64(string.getBytes("utf-8")));
    }

    /**
     *  Converst list of authorities to single string. Roles are separated by commas.
     * @param list list of authorities
     * @return single string
     */
    public static String authorityList(List<Authority> list)  {
        StringBuilder sb = new StringBuilder();
        if (list.isEmpty()) {
            return "";
        }
        for (Authority aut: list) {
            sb.append(aut.getRole());
            sb.append(", ");
        }
        return sb.substring(0,sb.length()-2);
    }
}


