package cz.muni.fi.mail;

import cz.muni.fi.exception.EmailSendException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * This class represents an email to send.
 *
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 3.11.11
 */
public class Email {

    private String from;

    private String fromName = "Internship Rating System";

    private String to;

    private String subject;

    private String body;

    /**
     * Whether this email should be sent as html. If true, the email content
     * must contain html tags.
     */
    private boolean html = false;

    protected final Log logger = LogFactory.getLog(getClass());

    public Email() {
    }

    public Email(String from, String to) {
        this.from = from;
        this.to = to;
        subject = "N/A";
        body = "empty body";
    }

    public Email(String s) {
        this.from = s;
    }

    public String getFrom() {
        return from;
    }

    public void setFrom(String from) {
        this.from = from;
    }

    public String getFromName() {
        return fromName;
    }

    public void setFromName(String fromName) {
        this.fromName = fromName;
    }

    public String getTo() {
        return to;
    }

    public void setTo(String to) throws EmailSendException {
        this.to = to;
        if (!checkEmailAddress()) {
            throw new EmailSendException("email address has wrong format");
        }
    }

    public String getSubject() {
        return subject;
    }

    public void setSubject(String subject) {
        this.subject = subject;
    }


    public String getBody() {
        return body;
    }

    public void setBody(String body) {
        this.body = body;
    }

    public boolean isHtml() {
        return html;
    }

    public void setHtml(boolean html) {
        this.html = html;
    }


    public String toString() {
        return "email: " + subject + " to " + to;
    }

    /**
     * Checks whether the recipient address is valid email.
     *
     * @return true if mail is valid, false otherwise
     */
    public boolean checkEmailAddress() {
        if (to == null) {
            return false;
        }
        return to.matches("^[\\w\\-]+(\\.[\\w\\-]+)*@([A-Za-z0-9-]+\\.)"
                + "+[A-Za-z]{2,4}$");
    }

}
