package cz.muni.fi.security;

import cz.muni.fi.domain.Period;
import cz.muni.fi.exception.DataIntegrityViolationException;
import cz.muni.fi.service.PeriodManager;
import cz.muni.fi.service.UserManager;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.userdetails.User;
import org.springframework.security.web.authentication.SavedRequestAwareAuthenticationSuccessHandler;
import org.springframework.stereotype.Component;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

/**
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 23.11.11
 */
@Component
public class AuthenticationSuccessHandlerImpl extends SavedRequestAwareAuthenticationSuccessHandler {

    @Autowired
    private UserManager userManager;

    @Autowired
    private PeriodManager periodManager;

    protected final Log logger = LogFactory.getLog(getClass());


    @Override
    public void onAuthenticationSuccess(HttpServletRequest request, HttpServletResponse response, Authentication authentication)
            throws IOException, ServletException {
        User current = (User) authentication.getPrincipal();
        String id = current.getUsername();
        Period period = userManager.getById(id).getSelectedPeriod();
        if (period == null) {
            try {
                period = periodManager.getCurrentPeriod();
            } catch (DataIntegrityViolationException e) {
                this.getRedirectStrategy().sendRedirect(request, response, "/noperiod");
                return;
            }
            logger.debug("No period selected, using current " + period);
            request.getSession().setAttribute("sessionPeriod", period);
            super.onAuthenticationSuccess(request, response, authentication);
        } else {
            logger.debug("Using selected period " + period);
            request.getSession().setAttribute("sessionPeriod", period);
            super.onAuthenticationSuccess(request, response, authentication);
        }
    }

}
