package cz.muni.fi.service;

import cz.muni.fi.domain.Feature;
import cz.muni.fi.domain.FeatureRate;
import cz.muni.fi.domain.Organization;
import cz.muni.fi.domain.Period;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 29.1.12
 */
@Service("aggregatedFeatureRateManager")
public class AggregatedFeatureRateManagerImpl implements AggregatedFeatureRateManager {
    @Autowired
    private PlainFeatureRateManager manager;

    @Autowired
    private FeatureManager featureManager;

    @Override
    public FeatureRate getRateForFeature(Feature feature, Period period) {
        FeatureRate rate = manager.getRateForFeature(feature, period);
        if (rate == null) {
            return null;
        }
        countGetRate(feature, period, rate);
        return rate;
    }

    private void countGetRate(Feature feature, Period period, FeatureRate rate) {
        if (feature.getSubFeatures() != null) {
            for (Feature sub : feature.getSubFeatures()) {
                FeatureRate subRate = manager.getRateForFeature(sub, period);
                if (subRate != null) {
                    rate.setResponses(rate.getResponses() + subRate.getResponses());
                    rate.setSum(rate.getSum() + subRate.getSum());
                    countGetRate(sub, period, rate);
                }
            }
        }
    }

    private void countGetRate(Feature feature, Organization org, FeatureRate rate) {
        if (feature.getSubFeatures() != null) {
            for (Feature sub : feature.getSubFeatures()) {
                FeatureRate subRate = manager.getRateForFeature(sub, org);
                if (subRate != null) {
                    rate.setResponses(rate.getResponses() + subRate.getResponses());
                    rate.setSum(rate.getSum() + subRate.getSum());
                    countGetRate(sub, org, rate);
                }
            }
        }
    }

    private void countGetRate(Feature feature, Period period, Organization org, FeatureRate rate) {
        if (feature.getSubFeatures() != null) {
            for (Feature sub : feature.getSubFeatures()) {
                FeatureRate subRate = manager.getRateForFeature(sub, period, org);
                if (subRate != null) {
                    rate.setResponses(rate.getResponses() + subRate.getResponses());
                    rate.setSum(rate.getSum() + subRate.getSum());
                    countGetRate(sub, period, org, rate);
                }
            }
        }
    }

    private void countGetRate(Feature feature, FeatureRate rate) {
        if (feature.getSubFeatures() != null) {
            for (Feature sub : feature.getSubFeatures()) {
                FeatureRate subRate = manager.getRateForFeature(sub);
                if (subRate != null) {
                    rate.setResponses(rate.getResponses() + subRate.getResponses());
                    rate.setSum(rate.getSum() + subRate.getSum());
                    countGetRate(sub, rate);
                }
            }
        }
    }

    @Override
    public FeatureRate getRateForFeature(Feature feature, Period period, Organization organization) {
        FeatureRate rate = manager.getRateForFeature(feature, period, organization);
        if (rate == null) {
            return null;
        }
        countGetRate(feature, period, organization, rate);
        return rate;
    }

    @Override
    public FeatureRate getRateForFeature(Feature feature) {
        FeatureRate rate = manager.getRateForFeature(feature);
        if (rate == null) {
            return null;
        }
        countGetRate(feature, rate);
        return rate;
    }

    @Override
    public FeatureRate getRateForFeature(Feature feature, Organization organization) {
        FeatureRate rate = manager.getRateForFeature(feature, organization);
        if (rate == null) {
            return null;
        }
        countGetRate(feature, organization, rate);
        return rate;
    }

    @Override
    public int getAverage() {
        List<FeatureRate> allRates = getAggregatedFeatureRates();
        return ((PlainFeatureRateManagerImpl) manager).countAverage(allRates);
    }

    @Override
    public int getAverage(Organization organization) {
        if (organization == null) {
            return getAverage();
        }
        List<FeatureRate> allRates = getAggregatedFeatureRates(organization);
        return ((PlainFeatureRateManagerImpl) manager).countAverage(allRates);
    }

    @Override
    public int getAverage(Period period) {
        if (period == null) {
            return getAverage();
        }
        List<FeatureRate> allRates = getAggregatedFeatureRates(period);
        return ((PlainFeatureRateManagerImpl) manager).countAverage(allRates);
    }

    @Override
    public int getAverage(Period period, Organization organization) {
        if (period == null) {
            return getAverage(organization);
        }
        if (organization == null) {
            return getAverage(period);
        }
        List<FeatureRate> allRates = getAllFeatureRates(period, organization);
        return ((PlainFeatureRateManagerImpl) manager).countAverage(allRates);
    }

    @Override
    public int getTotalImprovementCount(boolean negative) {
        List<FeatureRate> allRates = getAggregatedFeatureRates();
        int average = getAverage();
        return ((PlainFeatureRateManagerImpl) manager).countImprovement(true, allRates, average);
    }

    @Override
    public int getTotalImprovementCount(boolean negative, Organization organization) {
        if (organization == null) {
            return getTotalImprovementCount(negative);
        }
        List<FeatureRate> allRates = getAggregatedFeatureRates(organization);
        int average = getAverage(organization);
        return ((PlainFeatureRateManagerImpl) manager).countImprovement(true, allRates, average);

    }

    @Override
    public int getTotalImprovementCount(boolean negative, Period period) {
        if (period == null) {
            return getTotalImprovementCount(negative);
        }
        List<FeatureRate> allRates = getAggregatedFeatureRates(period);
        int average = getAverage(period);
        return ((PlainFeatureRateManagerImpl) manager).countImprovement(true, allRates, average);

    }

    @Override
    public int getTotalImprovementCount(boolean negative, Period period, Organization organization) {
        if (period == null) {
            return getTotalImprovementCount(negative, organization);
        }
        if (organization == null) {
            return getTotalImprovementCount(negative, period);
        }
        List<FeatureRate> allRates = getAllFeatureRates(period, organization);
        int average = getAverage(period, organization);
        return ((PlainFeatureRateManagerImpl) manager).countImprovement(true, allRates, average);
    }

    @Override
    public List<FeatureRate> getAggregatedFeatureRates(Period period) {
        if (period == null) {
            throw new IllegalArgumentException("period");
        }
        List<FeatureRate> result = new ArrayList<FeatureRate>();
        List<Feature> roots = featureManager.getAllRootFeatures();
        for (Feature feature : roots) {
            FeatureRate rate = getRateForFeature(feature, period);
            if (rate != null) {
                result.add(rate);
            }
        }
        return result;
    }

    @Override
    public List<FeatureRate> getAggregatedFeatureRates() {
        List<FeatureRate> result = new ArrayList<FeatureRate>();
        List<Feature> roots = featureManager.getAllRootFeatures();
        for (Feature feature : roots) {
            FeatureRate rate = getRateForFeature(feature);
            result.add(rate);
        }
        return result;
    }

    @Override
    public List<FeatureRate> getAggregatedFeatureRates(Organization organization) {
        if (organization == null) {
            throw new IllegalArgumentException("organization");
        }
        List<FeatureRate> result = new ArrayList<FeatureRate>();
        List<Feature> roots = featureManager.getAllRootFeatures();
        for (Feature feature : roots) {
            FeatureRate rate = getRateForFeature(feature, organization);
            result.add(rate);
        }
        return result;
    }

    @Override
    public List<FeatureRate> getAllFeatureRates(Period period, Organization organization) {
        if (period == null) {
            return getAggregatedFeatureRates(organization);
        }
        if (organization == null) {
            return getAggregatedFeatureRates(period);
        }
        List<FeatureRate> result = new ArrayList<FeatureRate>();
        List<Feature> roots = featureManager.getAllRootFeatures();
        for (Feature feature : roots) {
            FeatureRate rate = getRateForFeature(feature, period, organization);
            result.add(rate);
        }
        return result;
    }
}
