package cz.muni.fi.service;

import cz.muni.fi.domain.*;
import cz.muni.fi.exception.DataIntegrityViolationException;

import java.util.List;

/**
 * Manages forms and form pairs.
 *
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 9.8.11
 */
public interface FormManager {
    public FormPair createNewPair(Participant student, Participant manager, List<Feature> feature, Period period);

    public void saveForm(Form form);

    public FormPair findFormPair(Form form);

    public Form getFormByHash(String hash);

    public List<FormPair> getAllFormPairs();

    public List<FormPair> getAllFormPairs(Period period);

    public List<FormPair> getAllFormPairs(Period period, Organization organization);


    /**
     * Deleted form pair.
     * Form pair can be deleted only if it is new. (No form saved yet).
     *
     * @param pair pair
     * @throws DataIntegrityViolationException
     *          if form pair is not new
     */
    public void deleteFormPair(FormPair pair) throws DataIntegrityViolationException;

    /**
     * Deletes form pair and all its forms, regardless of the pair and forms state.
     *
     * @param pair pair
     */
    public void forceDeleteFormPair(FormPair pair);

    public FormPair getFormPairById(Long id);

    public boolean canBeAnswered(Form form) throws DataIntegrityViolationException;

    public void openForm(Form form);

    void enableFormReopen(Form form);

    Form getFormById(Long formId);

    void setFormPairTimestamp(FormPair pair);
}
