package cz.muni.fi.service;

import cz.muni.fi.domain.DisagreedFeatureRate;
import cz.muni.fi.domain.Organization;
import cz.muni.fi.domain.Participant;
import cz.muni.fi.exception.AlreadyPresentException;
import cz.muni.fi.exception.DataIntegrityViolationException;

import java.util.List;

/**
 * Manages features.
 *
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 9.8.11
 */
public interface ParticipantManager {

    public List<Participant> getAllStudents();

    public List<Participant> getAllManagers();

    public Participant getByEmail(String id);

    /**
     * Adds participant.
     *
     * @param participant participant to add, if manager, his organization must be already in db
     */
    public void addParticipant(Participant participant) throws AlreadyPresentException;

    public void deleteParticipant(Participant participant) throws DataIntegrityViolationException;

    boolean canParticipantBeDeleted(Participant participant);
    
    List<DisagreedFeatureRate> getDisagreementList(Participant participant);

    void updateParticipant(Participant participant) throws AlreadyPresentException;

    Participant getByUco(String studentUco);

    Participant getById(Long id);

    List<Participant> getAllManagersFor(Organization org);
}
