package cz.muni.fi.util;

import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

/**
 * Used to compute SHA256 digests.
 *
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 20.8.11
 */
public class HashGenerator {

    private static final String SHA_256 = "SHA-256";

    /**
     * Computes SHA256 digest of any String.
     *
     * @param word string value to compute hash from
     * @return hash as string
     * @throws NoSuchAlgorithmException if sha-256 can't be computed
     */
    public static String computeHash(String word) throws NoSuchAlgorithmException {
        if (word == null) {
            throw new IllegalArgumentException("word");
        }

        MessageDigest md = MessageDigest.getInstance(SHA_256);
        md.update(word.getBytes());

        byte byteData[] = md.digest();


        StringBuffer sb = new StringBuffer();
        for (byte aByteData : byteData) {
            sb.append(Integer.toString((aByteData & 0xff) + 0x100, 16).substring(1));
        }
        return sb.toString();

    }

    /**
     * Computes SHA-256 hash of Long value.
     *
     * @param id long value to compute hash from
     * @return sha256 hash, or other hash if sha256 can't be computed
     */
    public static String computeHash(Long id) {
        try {
            return computeHash(id.toString());
        } catch (NoSuchAlgorithmException ex) {
            return "3f5eW2g45" + id + "34FG" + id % 5;
        }
    }
}
