package cz.muni.fi.validation;

import cz.muni.fi.domain.Manager;
import cz.muni.fi.domain.Participant;
import cz.muni.fi.domain.Student;
import org.springframework.util.StringUtils;
import org.springframework.validation.BindingResult;

/**
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 4.11.11
 */
public class ParticipantValidator {


    public void validate(Participant participant, BindingResult result) {
        if (!StringUtils.hasLength(participant.getMail())) {
            result.rejectValue("mail", "required", "required");
        } else {
            if (!participant.getMail().matches(
                    "^[\\w\\-]+(\\.[\\w\\-]+)*@([A-Za-z0-9-]+\\.)+[A-Za-z]{2,4}$")) {
                result.rejectValue("mail", "bad.mail", "Invalid mail.");
            }
        }
        if (!StringUtils.hasLength(participant.getName())) {
            result.rejectValue("name", "required", "required");
        }
        if (!StringUtils.hasLength(participant.getSurname())) {
            result.rejectValue("surname", "required", "required");
        }
        if ((participant.getRole()) == null || !participant.getRole().matches("student|manager")) {
            result.rejectValue("role", "role.bad", "invalid role");
        }
        if (participant instanceof Manager) {
            if (((Manager) participant).getOrganization() == null) {
                result.rejectValue("organization", "required", "required");
            }
        }
        if (participant instanceof Student) {
            if (!StringUtils.hasLength(((Student) participant).getUco())) {
                result.rejectValue("uco", "required", "required");
            } else if (!((Student) participant).getUco().matches("\\d+")) {
                result.rejectValue("uco", "uco.bad", "bad uco format");
            }
        }
    }
}
