package cz.muni.fi.view;

import cz.muni.fi.domain.Feature;
import cz.muni.fi.domain.FeatureRating;
import cz.muni.fi.domain.FormPair;
import cz.muni.fi.util.ThresholdProvider;
import cz.muni.fi.wrapper.FeatureRateWrapper;
import org.apache.poi.hssf.usermodel.*;
import org.apache.poi.hssf.util.HSSFColor;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.ss.util.CellRangeAddress;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.web.servlet.view.document.AbstractExcelView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Locale;
import java.util.Map;

/**
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 14.1.12
 */
public class ExcelReportView extends AbstractExcelView {
    @Autowired
    private ApplicationContext context;

    @Autowired
    private ThresholdProvider thresholdProvider;

    @Override
    protected void buildExcelDocument(Map modelSent, HSSFWorkbook workbook,
                                      HttpServletRequest request, HttpServletResponse response)
            throws Exception {

        Map<String, Object> model = (Map<String, Object>) modelSent.get("model");
        String year = (String) model.get("year");
        if (year == null) {
            year = getMessage("year.global", request);
        }
        String headerMessage = getMessage("rates", request) + " - " + year;
        String headerMessageTotal = getMessage("rates.raw", request) + " - " + year;
        String org = ((model.get("organization") == null) ?
                getMessage("report.global", request) :
                (getMessage("report.organization", request) + " " + model.get("organization")));
        String exported = getMessage("report.exported", request) + " " + getDateAsString(request.getLocale());

        final List<FeatureRateWrapper> wrapperList = (List<FeatureRateWrapper>) model.get("totalRates");

        final List<FormPair> pairList = (List<FormPair>) model.get("formPairs");

        final List<Feature> features = (List<Feature>) model.get("features");

        //create a wordsheet
        String studSheetName = getMessage("xls.list.all.stud", request);
        String manSheetName = getMessage("xls.list.all.man", request);
        String avgSheetName = getMessage("xls.list.all.avg", request);

        HSSFSheet sheet = workbook.createSheet(getMessage("xls.list.rates", request));
        HSSFSheet sheetRawStud = workbook.createSheet(studSheetName);
        HSSFSheet sheetRawMan = workbook.createSheet(manSheetName);
        HSSFSheet sheetRawAvg = workbook.createSheet(avgSheetName);

        //Create header font
        Font font = workbook.createFont();
        font.setFontHeightInPoints((short) 12);
        font.setBoldweight(Font.BOLDWEIGHT_BOLD);
        HSSFCellStyle styleHeader = workbook.createCellStyle();
        styleHeader.setFont(font);

        Font fontBold = workbook.createFont();
        fontBold.setBoldweight(Font.BOLDWEIGHT_BOLD);
        HSSFCellStyle styleBold = workbook.createCellStyle();
        styleBold.setFont(fontBold);

        Font fontItalic = workbook.createFont();
        fontItalic.setItalic(true);
        HSSFCellStyle styleItalic = workbook.createCellStyle();
        styleItalic.setFont(fontItalic);

        //first sheet - feature rates
        HSSFCell hcell = sheet.createRow(0).createCell(0);
        hcell.setCellValue(headerMessage);
        hcell.setCellStyle(styleHeader);
        hcell = sheet.createRow(1).createCell(0);
        hcell.setCellValue(org);
        hcell.setCellStyle(styleHeader);
        hcell = sheet.createRow(2).createCell(0);
        hcell.setCellValue(exported);
        hcell.setCellStyle(styleHeader);

        mergeLine(sheet, 0, 0, 7);
        mergeLine(sheet, 1, 0, 7);
        mergeLine(sheet, 2, 0, 7);
        if (!wrapperList.isEmpty()) {
            HSSFRow header = sheet.createRow(4);
            hcell = header.createCell(0);
            hcell.setCellValue(getMessage("feature", request));
            hcell.setCellStyle(styleBold);
            header.createCell(1).setCellValue(getMessage("severity", request));

            int rowNum = 5;
            for (FeatureRateWrapper frw : wrapperList) {
                //create the row data
                HSSFRow row = sheet.createRow(rowNum++);
                HSSFCell cell = row.createCell(0);
                cell.setCellValue(frw.getFeature().getName());
                cell.setCellStyle(styleBold);
                row.createCell(1).setCellValue(frw.getValue());
            }

        } else {
            HSSFRow noDataRow = sheet.createRow(4);
            hcell = noDataRow.createCell(0);
            hcell.setCellValue(getMessage("no.data", request));
            hcell.setCellStyle(styleItalic);
        }
        //second sheet header - raw data for organization/period - student
        hcell = sheetRawStud.createRow(0).createCell(0);
        hcell.setCellValue(headerMessageTotal);
        hcell.setCellStyle(styleHeader);
        mergeLine(sheetRawStud, 0, 0, 5);
        hcell = sheetRawStud.createRow(1).createCell(0);
        hcell.setCellValue(org);
        hcell.setCellStyle(styleHeader);
        mergeLine(sheetRawStud, 1, 0, 5);
        hcell = sheetRawStud.createRow(2).createCell(0);
        hcell.setCellStyle(styleHeader);
        hcell.setCellValue(exported);
        mergeLine(sheetRawStud, 2, 0, 5);

        //third sheet header - raw data - manager
        hcell = sheetRawMan.createRow(0).createCell(0);
        hcell.setCellValue(headerMessageTotal);
        hcell.setCellStyle(styleHeader);
        mergeLine(sheetRawMan, 0, 0, 5);
        hcell = sheetRawMan.createRow(1).createCell(0);
        hcell.setCellValue(org);
        hcell.setCellStyle(styleHeader);
        mergeLine(sheetRawMan, 1, 0, 5);
        hcell = sheetRawMan.createRow(2).createCell(0);
        hcell.setCellStyle(styleHeader);
        hcell.setCellValue(exported);
        mergeLine(sheetRawMan, 2, 0, 5);

        //4th sheet header - raw data - average
        hcell = sheetRawAvg.createRow(0).createCell(0);
        hcell.setCellValue(headerMessageTotal);
        hcell.setCellStyle(styleHeader);
        mergeLine(sheetRawAvg, 0, 0, 5);
        hcell = sheetRawAvg.createRow(1).createCell(0);
        hcell.setCellValue(org);
        hcell.setCellStyle(styleHeader);
        mergeLine(sheetRawAvg, 1, 0, 5);
        hcell = sheetRawAvg.createRow(2).createCell(0);
        hcell.setCellStyle(styleHeader);
        hcell.setCellValue(exported);
        mergeLine(sheetRawAvg, 2, 0, 5);

        if (!pairList.isEmpty()) {
            HSSFRow rowStu = sheetRawStud.createRow(5);
            HSSFRow rowMan = sheetRawMan.createRow(5);

            HSSFRow rowAvg = sheetRawAvg.createRow(5);

            int fNum = 1;
            CreationHelper factory = workbook.getCreationHelper();

            Drawing drawingStu = sheetRawStud.createDrawingPatriarch();
            Drawing drawingMan = sheetRawMan.createDrawingPatriarch();
            Drawing drawingAvg = sheetRawAvg.createDrawingPatriarch();

            // When the comment box is visible, have it show in a 1x3 space
            ClientAnchor anchorStu = factory.createClientAnchor();
            ClientAnchor anchorMan = factory.createClientAnchor();
            ClientAnchor anchorAvg = factory.createClientAnchor();

            for (Feature feature : features) {
                hcell = rowStu.createCell(fNum);
                hcell.setCellValue(feature.getShortName());
                hcell.setCellStyle(styleBold);

                anchorStu.setCol1(hcell.getColumnIndex());
                anchorStu.setCol2(hcell.getColumnIndex()+5);
                anchorStu.setRow1(rowStu.getRowNum());
                anchorStu.setRow2(rowStu.getRowNum()+3);

                // Create the comment and set the text+author
                Comment commentStu = drawingStu.createCellComment(anchorStu);
                Comment commentMan = drawingMan.createCellComment(anchorMan);
                Comment commentAvg = drawingAvg.createCellComment(anchorAvg);

                RichTextString str = factory.createRichTextString(feature.getName());

                commentStu.setString(str);
                commentMan.setString(str);
                commentAvg.setString(str);

                // Assign the comment to the cell
                hcell.setCellComment(commentStu);


                hcell = rowMan.createCell(fNum);
                hcell.setCellValue(feature.getShortName());
                hcell.setCellStyle(styleBold);
                hcell.setCellComment(commentMan);

                anchorMan.setCol1(hcell.getColumnIndex());
                anchorMan.setCol2(hcell.getColumnIndex()+5);
                anchorMan.setRow1(rowMan.getRowNum());
                anchorMan.setRow2(rowMan.getRowNum()+3);


                hcell = rowAvg.createCell(fNum);
                hcell.setCellValue(feature.getShortName());
                hcell.setCellStyle(styleBold);
                hcell.setCellComment(commentAvg);

                anchorAvg.setCol1(hcell.getColumnIndex());
                anchorAvg.setCol2(hcell.getColumnIndex()+5);
                anchorAvg.setRow1(rowAvg.getRowNum());
                anchorAvg.setRow2(rowAvg.getRowNum()+3);

                fNum++;
            }
            int begLine = 6;
            HSSFCellStyle styleGreen = workbook.createCellStyle();
            styleGreen.setFillForegroundColor(new HSSFColor.BRIGHT_GREEN().getIndex());
            styleGreen.setFillPattern(CellStyle.SOLID_FOREGROUND);
            HSSFCellStyle styleRed = workbook.createCellStyle();
            styleRed.setFillForegroundColor(new HSSFColor.CORAL().getIndex());
            styleRed.setFillPattern(CellStyle.SOLID_FOREGROUND);
            for (FormPair pair : pairList) {
                if ("completed".equals(pair.getState())) {
                    HSSFRow studentRow = sheetRawStud.createRow(begLine);
                    HSSFRow managersRow = sheetRawMan.createRow(begLine);
                    HSSFRow avgRow = sheetRawAvg.createRow(begLine);
                    begLine++;
                    studentRow.createCell(0).setCellValue(pair.getStudentsForm().getOwner().getFullName());
                    managersRow.createCell(0).setCellValue(pair.getManagersForm().getOwner().getFullName());
                    HSSFCell avgCell = avgRow.createCell(0);
                    avgCell.setCellValue(pair.getManagersForm().getOwner().getFullName() + "/" + pair.getStudentsForm().getOwner().getFullName());
                    int index = 0;
                    for (FeatureRating rating : pair.getManagersForm().getFeatureRatingList()) {
                        int i = features.indexOf(rating.getFeature()) + 1;
                        int manValue = rating.getValue();
                        HSSFCell managerCell = managersRow.createCell(i);
                        managerCell.setCellValue(manValue);
                        int studValue = pair.getStudentsForm().getFeatureRatingList().get(index).getValue();
                        HSSFCell studentCell = studentRow.createCell(i);
                        studentCell.setCellValue(studValue);
                        index++;
                        String formula = getIfFormula(begLine, i + 1, studSheetName, manSheetName);
                        HSSFCell formulaCell = avgRow.createCell(i);
                        formulaCell.setCellFormula(formula);
                        if (Math.abs(manValue - studValue) >= thresholdProvider.getThreshold()) {
                            formulaCell.setCellStyle(styleRed);
                            studentCell.setCellStyle(styleRed);
                            managerCell.setCellStyle(styleRed);
                        }
                    }
                }
            }
            begLine++;
            HSSFRow studentTotalRow = sheetRawStud.createRow(begLine);
            HSSFRow managersTotalRow = sheetRawMan.createRow(begLine);
            HSSFRow avgTotalRow = sheetRawAvg.createRow(begLine);
            fNum = 1;
            for (Feature feature : features) {
                StringBuilder stForm = new StringBuilder("10*AVERAGE(");
                StringBuilder manForm = new StringBuilder("10*AVERAGE(");
                StringBuilder avgForm = new StringBuilder("10*AVERAGE(");

                stForm.append(getColumnLetter(fNum + 1));
                manForm.append(getColumnLetter(fNum + 1));
                avgForm.append(getColumnLetter(fNum + 1));
                stForm.append(7);
                manForm.append(7);
                avgForm.append(7);
                stForm.append(":");
                manForm.append(":");
                avgForm.append(":");

                stForm.append(getColumnLetter(fNum + 1));
                manForm.append(getColumnLetter(fNum + 1));
                avgForm.append(getColumnLetter(fNum + 1));
                stForm.append(begLine - 1);
                manForm.append(begLine - 1);
                avgForm.append(begLine - 1);
                stForm.append(")");
                manForm.append(")");
                avgForm.append(")");

                HSSFCell cell = avgTotalRow.createCell(fNum);
                cell.setCellFormula(avgForm.toString());
                cell.setCellStyle(styleBold);
                cell = avgTotalRow.createCell(0);
                cell.setCellValue(getMessage("average.total", request));
                cell.setCellStyle(styleBold);

                cell = studentTotalRow.createCell(fNum);
                cell.setCellFormula(stForm.toString());
                cell.setCellStyle(styleBold);
                cell = studentTotalRow.createCell(0);
                cell.setCellValue(getMessage("average.percent", request));
                cell.setCellStyle(styleBold);
                cell = managersTotalRow.createCell(fNum);
                cell.setCellFormula(manForm.toString());
                cell.setCellStyle(styleBold);
                cell = managersTotalRow.createCell(0);
                cell.setCellValue(getMessage("average.percent", request));
                cell.setCellStyle(styleBold);

                fNum++;
            }
        } else {
            HSSFRow noDataRow = sheetRawStud.createRow(4);
            hcell = noDataRow.createCell(0);
            hcell.setCellValue(getMessage("no.data", request));
            hcell.setCellStyle(styleItalic);
            noDataRow = sheetRawMan.createRow(4);
            hcell = noDataRow.createCell(0);
            hcell.setCellValue(getMessage("no.data", request));
            hcell.setCellStyle(styleItalic);
            noDataRow = sheetRawAvg.createRow(4);
            hcell = noDataRow.createCell(0);
            hcell.setCellValue(getMessage("no.data", request));
            hcell.setCellStyle(styleItalic);
        }

        sheetRawStud.autoSizeColumn(0, false);
        sheetRawMan.autoSizeColumn(0, false);
        sheetRawAvg.autoSizeColumn(0, false);

        sheet.autoSizeColumn(0, false);
        sheet.autoSizeColumn(1, false);

        DateFormat formatter = new SimpleDateFormat("yyyyMMddHHmmss");
        Date today = new Date();
        String dateString = formatter.format(today);
        String filename = "export_" + dateString + ".xls";
        response.addHeader("Content-Disposition", "attachment; filename=" + filename);
    }

    /**
     * Merges cells on line.
     *
     * @param sheetRaw sheet where to merge cells
     * @param rowI     row index
     * @param from     index of beginning merged cell
     * @param len      over how many cells to merge
     */
    private void mergeLine(HSSFSheet sheetRaw, int rowI, int from, int len) {
        sheetRaw.addMergedRegion(new CellRangeAddress(
                rowI, //first row (0-based)
                rowI, //last row  (0-based)
                from, //first column (0-based)
                from + len - 1  //last column  (0-based)
        ));
    }

    /**
     * Consturct if formula to count average or 0 if disagreement above threshold.
     *
     * @param line line number
     * @param col  column number
     * @return if formula as string
     */
    private String getIfFormula(int line, int col, String sname, String mname) {
        String colLetter = getColumnLetter(col);
        String cellNo = colLetter + String.valueOf(line);
        StringBuilder result = new StringBuilder("IF(ABS(");
        result.append("'");
        result.append(sname);
        result.append("'!");
        result.append(cellNo);
        result.append("-");
        result.append("'");
        result.append(mname);
        result.append("'!");
        result.append(cellNo);
        result.append(")>=");
        result.append(thresholdProvider.getThreshold());
        result.append(",\"\",((");
        result.append("'");
        result.append(sname);
        result.append("'!");
        result.append(cellNo);
        result.append("+");
        result.append("'");
        result.append(mname);
        result.append("'!");
        result.append(cellNo);
        result.append(")/2))");
        return result.toString();
    }

    /**
     * Converts column index to excel string - A,B,.. AA,AB..
     *
     * @param i column index
     * @return string column name
     */
    private String getColumnLetter(int i) {
        int iBase = 'Z' - 'A' + 1;
        String interConversion = Integer.toString(i - 1, iBase).toUpperCase();
        char[] ac = interConversion.toCharArray();
        for (int j = 0; j < ac.length; j++) {
            int posFromBack = ac.length - j - 1;
            char c = ac[j];
            ac[j] = (char) ('A' - posFromBack + Character.digit(c, iBase));
        }
        return String.copyValueOf(ac);
    }

    /**
     * Translates message from resource bundle
     *
     * @param key     message key
     * @param request servlet request (locale is taken from it)
     * @return localized message
     */
    private String getMessage(String key, HttpServletRequest request) {
        return context.getMessage(key, null, key, request.getLocale());
    }

    /**
     * @param currentLocale locale
     * @return date as string in current locale format
     */
    private String getDateAsString(Locale currentLocale) {
        DateFormat formatter = DateFormat.getDateInstance(DateFormat.DEFAULT,
                currentLocale);
        Date today = new Date();
        String output = formatter.format(today);
        return output;
    }

}
