package cz.muni.fi.view;

import cz.muni.fi.wrapper.FeatureRateWrapper;
import org.jfree.chart.ChartColor;
import org.jfree.chart.LegendItem;
import org.jfree.chart.LegendItemCollection;
import org.jfree.chart.LegendItemSource;

import java.awt.*;
import java.util.List;

/**
 * Generates legend for charts.
 *
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 21.10.11
 */
public class FeatureChartLegendSource implements LegendItemSource {
    private LegendItemCollection lic = new LegendItemCollection();

    /**
     * Constructor of legend source.
     *
     * @param wrapperList list of features to be included in the legend, must be sorted
     */
    public FeatureChartLegendSource(List<FeatureRateWrapper> wrapperList, boolean hints) {
        int i = 1;
        Paint[] paints = ChartColor.createDefaultPaintArray();
        for (FeatureRateWrapper frw : wrapperList) {
            StringBuilder legendS = new StringBuilder();
            legendS.append(i);
            legendS.append(": ");
            if (hints) {
                legendS.append(frw.getFeature().getHint());
            } else {
                legendS.append(frw.getFeature().getName());
            }
            legendS.append(" (");
            legendS.append(frw.getValue());
            legendS.append("%)");
            LegendItem li = new LegendItem(legendS.toString());
            li.setFillPaint(paints[(i - 1) % paints.length]);
            lic.add(li);
            i++;
        }
    }

    @Override
    public LegendItemCollection getLegendItems() {
        return lic;
    }
}
