package cz.muni.fi.web;

import cz.muni.fi.domain.Feature;
import cz.muni.fi.domain.Participant;
import cz.muni.fi.service.FeatureManager;
import cz.muni.fi.util.FeatureEditor;
import cz.muni.fi.util.ParticipantEditor;
import cz.muni.fi.validation.FeatureValidator;
import cz.muni.fi.wrapper.SessionMessageWrapper;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.bind.support.SessionStatus;

import javax.servlet.http.HttpSession;
import java.util.List;


/**
 * This is controller for feature adding form.
 *
 *
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 9.8.11
 */
@Controller
@RequestMapping("/features/new")
@SessionAttributes(types = Feature.class)
public class AddFeatureForm {

    private final FeatureManager featureManager;

    protected final Log logger = LogFactory.getLog(getClass());


    @Autowired
    public AddFeatureForm(FeatureManager manager) {
        this.featureManager = manager;
    }

    @InitBinder
    public void setAllowedFields(WebDataBinder dataBinder) {
        dataBinder.setDisallowedFields("id");
        dataBinder.registerCustomEditor(Feature.class, new FeatureEditor(featureManager));
    }

    @RequestMapping(method = RequestMethod.GET)
    public String setupForm(HttpSession session,Model model) {
        logger.debug("Rendering form for add feature.");
        session.removeAttribute("sessionMessage");
        Feature feature = new Feature();
        model.addAttribute(feature);
        return "features/form";
    }

    @RequestMapping(method = RequestMethod.POST)
    public String processSubmit(HttpSession session,@ModelAttribute Feature feature, BindingResult result, SessionStatus status) {
       logger.debug("Saving form for add feature.");
        session.removeAttribute("sessionMessage");
        new FeatureValidator().validate(feature, result);
        if (result.hasErrors()) {
            logger.debug("Form validation failed " + result.getAllErrors());
            return "features/form";
        } else {
            this.featureManager.addFeature(feature);
            status.setComplete();
            session.setAttribute("sessionMessage", new SessionMessageWrapper("okMessage", "successfully.saved"));
            return "redirect:/features/list";
        }
    }

    @ModelAttribute("featureParentList")
    public List<Feature> getFeatureList() {
        return featureManager.getAllRootFeatures();
    }


}
