package cz.muni.fi.web;

import cz.muni.fi.domain.Manager;
import cz.muni.fi.domain.Organization;
import cz.muni.fi.exception.AlreadyPresentException;
import cz.muni.fi.service.OrganizationManager;
import cz.muni.fi.service.ParticipantManager;
import cz.muni.fi.util.OrganizationEditor;
import cz.muni.fi.validation.ParticipantValidator;
import cz.muni.fi.wrapper.SessionMessageWrapper;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.bind.support.SessionStatus;
import org.springframework.web.servlet.view.RedirectView;

import javax.servlet.http.HttpSession;
import java.util.List;

/**
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 8.11.11
 */

@Controller
@RequestMapping("/managers/new")
@SessionAttributes(types = Manager.class, value = {"newManager", "canEditOrg"})
public class AddManagerForm {

    private final ParticipantManager participantManager;

    private final OrganizationManager organizationManager;

    protected final Log logger = LogFactory.getLog(getClass());


    @Autowired
    public AddManagerForm(ParticipantManager manager, OrganizationManager organizationManager) {
        this.participantManager = manager;
        this.organizationManager = organizationManager;
    }

    @InitBinder
    public void setAllowedFields(WebDataBinder dataBinder) {
        dataBinder.setDisallowedFields("role", "id");
        dataBinder.registerCustomEditor(Organization.class, new OrganizationEditor(organizationManager));
    }

    @RequestMapping(method = RequestMethod.GET)
    public String setupForm(Model model) {
        logger.debug("Rendering form to add manager");
        Manager manager = new Manager();
        manager.setRole("manager");
        model.addAttribute(manager);
        model.addAttribute("newManager", true);
        model.addAttribute("canEditOrg", true);
        return "organizations/manager";
    }

    @RequestMapping(method = RequestMethod.POST)
    public Object processSubmit(HttpSession session, @ModelAttribute Manager manager, BindingResult result, SessionStatus status) {
        session.removeAttribute("sessionMessage");
        logger.debug("Going to save add manager form.");
        new ParticipantValidator().validate(manager, result);
        if (result.hasErrors()) {
            logger.debug("Form validation failed.");
            return "organizations/manager";
        } else {
            try {
                this.participantManager.addParticipant(manager);
            } catch (AlreadyPresentException e) {
                session.setAttribute("sessionMessage", new SessionMessageWrapper("errorMessage", "participant.already.present"));
            }
            if (session.getAttribute("sessionMessage") == null) {
                session.setAttribute("sessionMessage", new SessionMessageWrapper("okMessage", "successfully.saved"));
            }
            status.setComplete();
            return redirect("/organizations/list");
        }
    }

    @ModelAttribute("allOrganizations")
    public List<Organization> populateOrganizations() {
        return organizationManager.getAllActiveOrganizations();
    }

    private RedirectView redirect(String url) {
        RedirectView view = new RedirectView(url, true);
        view.setExposeModelAttributes(false);
        return view;
    }
}
