package cz.muni.fi.web;

import cz.muni.fi.domain.Organization;
import cz.muni.fi.exception.AlreadyPresentException;
import cz.muni.fi.service.OrganizationManager;
import cz.muni.fi.validation.OrganizationValidator;
import cz.muni.fi.wrapper.SessionMessageWrapper;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.bind.support.SessionStatus;
import org.springframework.web.servlet.view.RedirectView;

import javax.servlet.http.HttpSession;

/**
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 8.11.11
 */

@Controller
@RequestMapping("/organizations/new")
@SessionAttributes(types = Organization.class,value = "newOrganization")
public class AddOrganizationForm {

    private final OrganizationManager organizationManager;

    protected final Log logger = LogFactory.getLog(getClass());


    @Autowired
    public AddOrganizationForm(OrganizationManager organizationManager) {
        this.organizationManager = organizationManager;
    }

    @InitBinder
    public void setAllowedFields(WebDataBinder dataBinder) {
        dataBinder.setDisallowedFields("id");
    }

    @RequestMapping(method = RequestMethod.GET)
    public String setupForm(Model model, HttpSession session) {
        logger.debug("Rendering add organization form");
        session.removeAttribute("sessionMessage");
        Organization organization = new Organization();
        model.addAttribute(organization);
        model.addAttribute("newOrganization",true);
        return "organizations/form";
    }

    @RequestMapping(method = RequestMethod.POST)
    public Object processSubmit(HttpSession session, @ModelAttribute Organization organization, BindingResult result, SessionStatus status) {
        logger.debug("Saving add organization form");
        new OrganizationValidator().validate(organization, result);
        if (result.hasErrors()) {
            logger.debug("Form validation failed");
            return "organizations/form";
        } else {
            organization.setActive(true);
            try {
                this.organizationManager.addOrganization(organization);
            } catch (AlreadyPresentException e) {
                logger.info("Organization " + organization + " already present.");
                session.setAttribute("sessionMessage", new SessionMessageWrapper("errorMessage", "organization.already.present"));
                status.setComplete();
                return redirect("/organizations/list");
            }
            status.setComplete();
            session.setAttribute("sessionMessage", new SessionMessageWrapper("okMessage", "successfully.saved"));
            return redirect("/organizations/list");
        }
    }

    private RedirectView redirect(String url) {
        RedirectView view = new RedirectView(url, true);
        view.setExposeModelAttributes(false);
        return view;
    }
}
