package cz.muni.fi.web;

import cz.muni.fi.domain.Period;
import cz.muni.fi.service.PeriodManager;
import cz.muni.fi.validation.PeriodValidator;
import cz.muni.fi.wrapper.SessionMessageWrapper;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.bind.support.SessionStatus;

import javax.servlet.http.HttpSession;

/**
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 24.11.11
 */

@Controller
@RequestMapping("/periods/new")
@SessionAttributes(types = Period.class)
public class AddPeriodForm {
    private final PeriodManager periodManager;

    protected final Log logger = LogFactory.getLog(getClass());


    @Autowired
    public AddPeriodForm(PeriodManager periodManager) {
        this.periodManager = periodManager;
    }

    @InitBinder
    public void setAllowedFields(WebDataBinder dataBinder) {
        dataBinder.setDisallowedFields("id", "timestamp");
    }

    @RequestMapping(method = RequestMethod.GET)
    public String setupForm(HttpSession session, Model model) {
        logger.debug("Rendering add period form");
        session.removeAttribute("sessionMessage");
        Period period = new Period();
        model.addAttribute(period);
        return "periods/form";
    }

    @RequestMapping(method = RequestMethod.POST)
    public String processSubmit(HttpSession session, @ModelAttribute Period period, BindingResult result, SessionStatus status) {
      logger.debug("Saving add period form");
        session.removeAttribute("sessionMessage");
        new PeriodValidator().validate(period, result);
        if (result.hasErrors()) {
            logger.debug("Validation failed.");
            return "periods/form";
        } else {
            this.periodManager.addPeriod(period);
            status.setComplete();
            if (session.getAttribute("sessionPeriod") == null) {
                session.setAttribute("sessionPeriod", period);
            }
            session.setAttribute("sessionMessage", new SessionMessageWrapper("okMessage", "successfully.saved"));
            return "redirect:/periods/list";
        }
    }

}
