package cz.muni.fi.web;

import cz.muni.fi.domain.Participant;
import cz.muni.fi.domain.Student;
import cz.muni.fi.exception.AlreadyPresentException;
import cz.muni.fi.service.ParticipantManager;
import cz.muni.fi.validation.ParticipantValidator;
import cz.muni.fi.wrapper.SessionMessageWrapper;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.bind.support.SessionStatus;
import org.springframework.web.servlet.view.RedirectView;

import javax.servlet.http.HttpSession;

/**
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 4.11.11
 */

@Controller
@RequestMapping("/students/new")
@SessionAttributes(types = Student.class,value="newStudent")
public class AddStudentForm {

    private final ParticipantManager participantManager;

    protected final Log logger = LogFactory.getLog(getClass());


    @Autowired
    public AddStudentForm(ParticipantManager manager) {
        this.participantManager = manager;
    }

    @InitBinder
    public void setAllowedFields(WebDataBinder dataBinder) {
        dataBinder.setDisallowedFields("role","id");
    }

    @RequestMapping(method = RequestMethod.GET)
    public String setupForm(HttpSession session, Model model) {
        logger.debug("Rendering add student form");
        session.removeAttribute("sessionMessage");
        Student student = new Student();
        student.setRole("student");
        model.addAttribute(student);
        model.addAttribute("newStudent",true);
        return "students/form";
    }

    @RequestMapping(method = RequestMethod.POST)
    public Object processSubmit(HttpSession session, @ModelAttribute Student student, BindingResult result, SessionStatus status) {
     logger.debug("Saving add student form");
        new ParticipantValidator().validate(student, result);
        if (result.hasErrors()) {
            logger.debug("Form validation failed");
            return "students/form";
        } else {
            try {
                this.participantManager.addParticipant(student);
            } catch (AlreadyPresentException e) {
                session.setAttribute("sessionMessage", new SessionMessageWrapper("errorMessage", "participant.already.present"));
            }
            status.setComplete();
            if (session.getAttribute("sessionMessage") == null) {
                session.setAttribute("sessionMessage", new SessionMessageWrapper("okMessage", "successfully.saved"));
            }
            return redirect("/students/list");
        }
    }
    private RedirectView redirect(String url) {
        RedirectView view = new RedirectView(url, true);
        view.setExposeModelAttributes(false);
        return view;
    }
}
