package cz.muni.fi.web;

import cz.muni.fi.domain.Authority;
import cz.muni.fi.domain.User;
import cz.muni.fi.service.UserManager;
import cz.muni.fi.util.HashGenerator;
import cz.muni.fi.validation.PasswordValidator;
import cz.muni.fi.validation.UserValidator;
import cz.muni.fi.wrapper.SessionMessageWrapper;
import cz.muni.fi.wrapper.UserAuthorityWrapper;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.bind.support.SessionStatus;
import org.springframework.web.servlet.view.RedirectView;

import javax.servlet.http.HttpSession;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;

/**
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 28.10.11
 */
@Controller
@SessionAttributes(types = UserAuthorityWrapper.class, value = "newUser")
public class AddUserForm {

    private UserManager userManager;

    protected final Log logger = LogFactory.getLog(getClass());

    @Autowired
    public AddUserForm(UserManager userManager) {
        this.userManager = userManager;
    }

    @InitBinder
    public void setAllowedFields(WebDataBinder dataBinder) {
        // dataBinder.setDisallowedFields("id");
    }

    @RequestMapping(value = "/admin/new", method = RequestMethod.GET)
    public String setupForm(Model model, HttpSession session) {
        logger.debug("Rendering add user form");
        session.removeAttribute("sessionMessage");
        UserAuthorityWrapper userAuthorityWrapper = new UserAuthorityWrapper();
        User user = new User();
        user.setAuthorities(new ArrayList<Authority>());
        Authority authority = new Authority();
        userAuthorityWrapper.setUser(user);
        userAuthorityWrapper.setAuthority(authority);
        userAuthorityWrapper.setAvailableRoles(userManager.getAvailableRoles());
        model.addAttribute(userAuthorityWrapper);
        model.addAttribute("newUser", true);
        return "admin/form";
    }

    @RequestMapping(value = "/admin/new", method = RequestMethod.POST)
    public Object processSubmit(HttpSession session, @ModelAttribute UserAuthorityWrapper userAuthorityWrapper, BindingResult result, SessionStatus status) {
        logger.debug("Saving add user form");
        session.removeAttribute("sessionMessage");
        if (!"NONE".equals(userAuthorityWrapper.getAuthority().getRole())) {
            logger.debug("Adding role for user.");
            userAuthorityWrapper.getUser().getAuthorities().add(userAuthorityWrapper.getAuthority());
            userAuthorityWrapper.getAvailableRoles().remove(userAuthorityWrapper.getAuthority().getRole());
            userAuthorityWrapper.setAuthority(new Authority());
            userAuthorityWrapper.setPassword(userAuthorityWrapper.getPassword());
            return "admin/form";
        }
        new UserValidator().validate(userAuthorityWrapper.getUser(), result);
        if (result.hasErrors()) {
            logger.debug("Form validation failed");
            return "admin/form";
        }
        new PasswordValidator().validate(userAuthorityWrapper.getPassword(), userAuthorityWrapper.getPasswordConfirm(), result);
        if (result.hasErrors()) {
            logger.debug("Password form validation failed");
            return "admin/form";
        } else {
            try {
                userAuthorityWrapper.getUser().setPassword(HashGenerator.computeHash(userAuthorityWrapper.getPassword()));
            } catch (NoSuchAlgorithmException e) {
                logger.error("En exception happened while trying to generate password hash.", e);
                return "error";
            }
            this.userManager.addUser(userAuthorityWrapper.getUser());
            session.setAttribute("sessionMessage", new SessionMessageWrapper("okMessage", "successfully.saved"));
            status.setComplete();
            return redirect("/admin");
        }
    }

    @RequestMapping("/admin/new/delete/{roleId}")
    public Object deleteRoleHandler
            (@ModelAttribute UserAuthorityWrapper userAuthorityWrapper,
             @PathVariable("roleId") String roleName,
             SessionStatus status, HttpSession session) {
        logger.debug("Deleting new user role.");
        session.removeAttribute("sessionMessage");
        if (!userAuthorityWrapper.getUser().getAuthorities().contains(roleName)) {
            logger.info("User does not have role " + roleName + " -  will not be removed.");
        } else {
            userAuthorityWrapper.getUser().getAuthorities().remove(roleName);
            userAuthorityWrapper.getAvailableRoles().add(roleName);
        }
        return redirect("/admin/new ");
    }

    private RedirectView redirect(String url) {
        RedirectView view = new RedirectView(url, true);
        view.setExposeModelAttributes(false);
        return view;
    }

}
