package cz.muni.fi.web;

import cz.muni.fi.domain.User;
import cz.muni.fi.service.UserManager;
import cz.muni.fi.wrapper.SessionMessageWrapper;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.ServletException;
import javax.servlet.http.HttpSession;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

/**
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 27.10.11
 */
@Controller
public class AdminController {

    protected final Log logger = LogFactory.getLog(getClass());

    private final UserManager userManager;

    @Autowired
    public AdminController(UserManager manager) {
        this.userManager = manager;
    }

    @RequestMapping("/admin")
    public ModelAndView adminMenuHandler(HttpSession session)
            throws ServletException, IOException {
        logger.debug("Rendering root admin view");
        SessionMessageWrapper atr = (SessionMessageWrapper) session.getAttribute("sessionMessage");
        if (atr != null) {
            if (atr.isSeen()) {
                session.removeAttribute("sessionMessage");
            } else {
                atr.setSeen(true);
            }
        }
        Map<String, Object> model = new HashMap<String, Object>();
        model.put("users", userManager.getAllUsers());
        return new ModelAndView("admin/home", "model", model);

    }


    @RequestMapping("/admin/delete/{userId}")
    public ModelAndView userDeleteHandler(HttpSession session, @PathVariable("userId") String userId)
            throws ServletException, IOException {
        logger.debug("Going to delete user");
        session.removeAttribute("sessionMessage");
        User user = userManager.getById(userId);
        if (user == null) {
            logger.warn("User " + userId + " not found, was not deleted.");
            session.setAttribute("sessionMessage", new SessionMessageWrapper("errorMessage", "user.not.found"));
        } else {
            if (userManager.getAllUsers().size() < 2) {
                logger.warn("Last user can't be deleted.");
                session.setAttribute("sessionMessage", new SessionMessageWrapper("errorMessage", "user.cannot.delete"));
            } else {
                userManager.deleteUser(user);
                session.setAttribute("sessionMessage", new SessionMessageWrapper("okMessage", "successfully.deleted"));
            }
        }
        Map<String, Object> model = new HashMap<String, Object>();
        model.put("users", userManager.getAllUsers());
        return new ModelAndView("redirect:/admin", "model", model);

    }


}
