package cz.muni.fi.web;

import cz.muni.fi.domain.Feature;
import cz.muni.fi.domain.Participant;
import cz.muni.fi.service.FeatureManager;
import cz.muni.fi.service.ParticipantManager;
import cz.muni.fi.wrapper.SessionMessageWrapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;

import javax.servlet.http.HttpSession;

/**
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 10.3.12
 */
@Controller
public class DisagreementController {
    private final FeatureManager featureManager;


    private final ParticipantManager participantManager;

    @Autowired
    public DisagreementController(ParticipantManager participantManager, FeatureManager featureManager) {
        this.participantManager = participantManager;
        this.featureManager = featureManager;
    }

    @RequestMapping("/disagreements/feature/{featureId}")
    public String getFeatureDisagreements(HttpSession session, Model model, @PathVariable("featureId") Long featureId) {
        SessionMessageWrapper atr = (SessionMessageWrapper) session.getAttribute("sessionMessage");
        session.removeAttribute("sessionMessage");
        Feature feature = featureManager.getFeatureById(featureId);
        if (feature == null) {
            SessionMessageWrapper message = new SessionMessageWrapper("errorMessage", "feature.not.found");
            message.setSeen(true);
            session.setAttribute("sessionMessage", message);

        } else {
            model.addAttribute("list", featureManager.getDisagreementList(feature));
        }
        model.addAttribute("subject", feature);
        return "disagreements/feature";
    }


    @RequestMapping("/disagreements/participant/{partId}")
    public String getParticipantDisagreements(HttpSession session, Model model, @PathVariable("partId") Long partId) {
        SessionMessageWrapper atr = (SessionMessageWrapper) session.getAttribute("sessionMessage");
        session.removeAttribute("sessionMessage");
        Participant participant = participantManager.getById(partId);
        if (participant == null) {
            SessionMessageWrapper message = new SessionMessageWrapper("errorMessage", "user.not.found");
            message.setSeen(true);
            session.setAttribute("sessionMessage", message);
        } else {
            model.addAttribute("list", participantManager.getDisagreementList(participant));
        }
        model.addAttribute("subject", participant);
        return "disagreements/list";
    }

}
