package cz.muni.fi.web;

import cz.muni.fi.domain.Feature;
import cz.muni.fi.service.FeatureManager;
import cz.muni.fi.util.FeatureEditor;
import cz.muni.fi.validation.FeatureValidator;
import cz.muni.fi.wrapper.SessionMessageWrapper;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.bind.support.SessionStatus;

import javax.servlet.http.HttpSession;
import java.util.ArrayList;
import java.util.List;

/**
 * Form controller for editing features.
 *
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 9.8.11
 */
@Controller
@RequestMapping("/features/{featureId}/edit")
@SessionAttributes(types = Feature.class)
public class EditFeatureForm {

    private final FeatureManager featureManager;

    protected final Log logger = LogFactory.getLog(getClass());

    @Autowired
    public EditFeatureForm(FeatureManager manager) {
        this.featureManager = manager;
    }

    @InitBinder
    public void setAllowedFields(WebDataBinder dataBinder) {
        dataBinder.setDisallowedFields("id");
        dataBinder.registerCustomEditor(Feature.class, new FeatureEditor(featureManager));

    }

    @RequestMapping(method = RequestMethod.GET)
    public String setupForm(HttpSession session, @PathVariable("featureId") Long featureId, Model model) {
        logger.debug("Rendering edit feature form");
        session.removeAttribute("sessionMessage");
        Feature feature = this.featureManager.getFeatureById(featureId);
        if (feature == null) {
            logger.info("No such feature " + featureId);
            session.setAttribute("sessionMessage", new SessionMessageWrapper("errorMessage", "feature.not.found"));
            return "redirect:/features/list";
        }
        model.addAttribute(feature);
        if ((!feature.getSubFeatures().isEmpty()) && (feature.getParent() == null)) {
            model.addAttribute("featureParentList",new ArrayList<Feature>());
        }  else {
            List<Feature> parents= featureManager.getAllRootFeatures();
            parents.remove(feature);
            model.addAttribute("featureParentList",parents);
        }
        return "features/form";
    }

    @RequestMapping(method = RequestMethod.PUT)
    public String processSubmit(HttpSession session,@ModelAttribute Feature feature, BindingResult result, SessionStatus status) {
        logger.debug("Saving edit feature form.");
        session.removeAttribute("sessionMessage");
        new FeatureValidator().validate(feature, result);
        if (result.hasErrors()) {
            logger.debug("Validation failed");
            return "features/form";
        } else {
            this.featureManager.updateFeature(feature);
            status.setComplete();
            session.setAttribute("sessionMessage", new SessionMessageWrapper("okMessage", "successfully.saved"));
            return "redirect:/features/list";
        }
    }

}
