package cz.muni.fi.web;

import cz.muni.fi.domain.Feature;
import cz.muni.fi.service.FeatureManager;
import cz.muni.fi.wrapper.FeatureWrapper;
import cz.muni.fi.wrapper.SessionMessageWrapper;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.ServletException;
import javax.servlet.http.HttpSession;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Features controller.
 *
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 9.8.11
 */
@Controller
public class FeatureController {

    protected final Log logger = LogFactory.getLog(getClass());

    private final FeatureManager featureManager;

    @Autowired
    public FeatureController(FeatureManager manager) {
        this.featureManager = manager;
    }

    @RequestMapping("/features/list")
    public ModelAndView featureListHandler(HttpSession session)
            throws ServletException, IOException {
        logger.debug("Rendering features list.");
        SessionMessageWrapper atr = (SessionMessageWrapper) session.getAttribute("sessionMessage");
        if (atr != null) {
            if (atr.isSeen()) {
                session.removeAttribute("sessionMessage");
            } else {
                atr.setSeen(true);
            }
        }
        logger.info("Returning list of features");
        Map<String, Object> myModel = new HashMap<String, Object>();
        constructFeatures(myModel);
        return new ModelAndView("features/list", "model", myModel);

    }

    private void constructFeatures(Map<String, Object> myModel) {
        List<Feature> features = this.featureManager.getAllRootFeatures();
        List<FeatureWrapper> wrapperList = new ArrayList<FeatureWrapper>();
        for (Feature f : features) {
            FeatureWrapper wrapper = new FeatureWrapper();
            wrapper.setFeature(f);
            wrapper.setCanBeDeleted(featureManager.canFeatureBeDeleted(f));
            wrapperList.add(wrapper);
        }
        myModel.put("features", wrapperList);
    }

    @RequestMapping("/features/{featureId}")
    public Object ownerHandler(HttpSession session, @PathVariable("featureId") Long featureId) {
        session.removeAttribute("sessionMessage");
        logger.debug("Rendering feature detail.");
        Feature feature = this.featureManager.getFeatureById(featureId);
        if (feature == null) {
            logger.warn("No such feature " + featureId);
            session.setAttribute("sessionMessage", new SessionMessageWrapper("errorMessage", "feature.not.found"));
            return "redirect:/features/list";
        }
        ModelAndView mav = new ModelAndView("features/show");
        mav.addObject(feature);
        int dis = featureManager.getDisagreementList(feature).size();
        mav.addObject("disagreements",dis);
        return mav;
    }

    @RequestMapping("/features/delete/{featureId}")
    public String featureDeleteHandler(HttpSession session, @PathVariable("featureId") Long featureId, Model model)
            throws ServletException, IOException {
        logger.debug("Going to delete feature " + featureId);
        session.removeAttribute("sessionMessage");
        Feature feature = featureManager.getFeatureById(featureId);
        if (feature == null) {
            logger.error("Feature " + featureId + " was not found.");
            session.setAttribute("sessionMessage", new SessionMessageWrapper("errorMessage", "feature.not.found"));
        } else {
            if (!featureManager.canFeatureBeDeleted(feature)) {
                logger.warn("Feature " + feature + " cannot be deleted.");
                session.setAttribute("sessionMessage", new SessionMessageWrapper("errorMessage", "feature.cannot.delete"));
            } else {
                featureManager.deleteFeature(feature);
                logger.info("Feature " + feature + "deleted.");
                session.setAttribute("sessionMessage", new SessionMessageWrapper("okMessage", "successfully.deleted"));
            }
        }
        return "redirect:/features/list";
    }

}