package cz.muni.fi.web;

import cz.muni.fi.domain.Form;
import cz.muni.fi.domain.Manager;
import cz.muni.fi.service.FormManager;
import cz.muni.fi.service.FormManagerImpl;
import cz.muni.fi.wrapper.SessionMessageWrapper;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.bind.support.SessionStatus;

import javax.servlet.http.HttpSession;
import java.util.Date;

/**
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 12.11.11
 */

@Controller
@RequestMapping("/forms/{formId}")
@SessionAttributes(types = Form.class)

public class FormDetailForm {

    protected final Log logger = LogFactory.getLog(getClass());

    private final FormManager formManager;

    @Autowired
    public FormDetailForm(FormManager manager) {
        this.formManager = manager;
    }

    @InitBinder
    public void setAllowedFields(WebDataBinder dataBinder) {
        dataBinder.setDisallowedFields("id");
    }

    @RequestMapping(method = RequestMethod.GET)
    public String setupForm(HttpSession session, @PathVariable("formId") Long formId, Model model) {
        logger.debug("Rendering form detail form for form " + formId + ".");
        session.removeAttribute("sessionMessage");

        Form form = this.formManager.getFormById(formId);
        if (form == null) {
            session.setAttribute("sessionMessage", new SessionMessageWrapper("errorMessage", "form.not.found"));
            return "redirect:/forms/list";
        }
        model.addAttribute(form);
        if ("opened".equals(form.getState())) {
            long now = new Date().getTime();
            if ((now - form.getTimestamp().getTime()) > (FormManagerImpl.FORM_TIMEOUT_SECONDS * 1000)) {
                model.addAttribute("expired", true);
            }
        }
        if (form.getOwner() instanceof Manager) {
            model.addAttribute("isManager", true);
        }
        return "forms/show";
    }

    @RequestMapping(method = RequestMethod.POST)
    public String processSubmit(HttpSession session, @ModelAttribute Form form, SessionStatus status) {
        logger.debug("Saving form detail form.");
        session.removeAttribute("sessionMessage");
        this.formManager.enableFormReopen(form);
        status.setComplete();
        return "redirect:/forms/" + form.getId();
    }
}


