package cz.muni.fi.web;


import cz.muni.fi.service.UserManager;
import cz.muni.fi.util.HashGenerator;
import cz.muni.fi.validation.PasswordValidator;
import cz.muni.fi.wrapper.UserAuthorityWrapper;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.SessionAttributes;
import org.springframework.web.bind.support.SessionStatus;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.view.RedirectView;

import javax.servlet.http.HttpSession;
import java.security.NoSuchAlgorithmException;
import java.security.Principal;
import java.util.HashMap;
import java.util.Map;

/**
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 24.10.11
 */
@Controller
@SessionAttributes(types = UserAuthorityWrapper.class, value = {"currentUser"})
public class LoginController {

    @Autowired
    private UserManager userManager;

    protected final Log logger = LogFactory.getLog(getClass());

    @RequestMapping("/login")
    public String loginHandler(Principal principal) {
        logger.debug("Rendering login page.");
        return "login";
    }

    @RequestMapping("/loginerror")
    public ModelAndView loginErroHandler() {
        logger.debug("Logging in was not successfull, rendering error.");
        Map<String, Object> myModel = new HashMap<String, Object>();
        myModel.put("error", true);
        return new ModelAndView("login", "model", myModel);
    }

    @RequestMapping("/logoutsuccess")
    public ModelAndView logoutSuccessHandler() {
        logger.debug("Successfully logged out.");
        Map<String, Object> myModel = new HashMap<String, Object>();
        myModel.put("logout", true);
        return new ModelAndView("login", "model", myModel);
    }

    @RequestMapping("/logout")
    public String logoutHandler() {
        logger.debug("Logging out.");
        return "redirect:/j_spring_security_logout";
    }

    @RequestMapping(method = RequestMethod.GET, value = "/myaccount")
    public String myAccountSetup(Model model, Principal principal) {
        logger.debug("Rendering my account form.");
        String name = principal.getName();
        model.addAttribute("currentUser", userManager.getById(name));
        model.addAttribute("userAuthorityWrapper", new UserAuthorityWrapper());
        model.addAttribute("newForm");
        return "myaccount";
    }

    @RequestMapping(method = RequestMethod.POST, value = "/myaccount")
    public Object myAccountSubmit(Principal principal, @ModelAttribute UserAuthorityWrapper userAuthorityWrapper
            , BindingResult result, SessionStatus status, HttpSession session) {
       logger.debug("Saving my account form");
        String name = principal.getName();
        new PasswordValidator().validate(userAuthorityWrapper.getPassword(), userAuthorityWrapper.getPasswordConfirm(), result);
        if (result.hasErrors()) {
            logger.debug("Form validation failed");
            return "myaccount";
        } else {
            String pass = "";
            try {
                pass = HashGenerator.computeHash(userAuthorityWrapper.getPassword());
            } catch (NoSuchAlgorithmException e) {
                logger.error("En exception happened while trying to generate password hash.", e);
                return "error";
            }
            String old = userManager.getById(name).getPassword();
            userManager.changePassword(principal.getName(), old, pass);
            status.setComplete();
            session.setAttribute("info", "saved.success");
            RedirectView view = new RedirectView("/",true);
            view.setExposeModelAttributes(false);
            return view;
        }
    }

}
