package cz.muni.fi.web;

import cz.muni.fi.domain.*;
import cz.muni.fi.exception.EmailSendException;
import cz.muni.fi.mail.MailCommons;
import cz.muni.fi.service.FeatureManager;
import cz.muni.fi.service.FormManager;
import cz.muni.fi.service.ParticipantManager;
import cz.muni.fi.service.UserManager;
import cz.muni.fi.util.*;
import cz.muni.fi.validation.FormPairValidator;
import cz.muni.fi.validation.ListFormPairValidator;
import cz.muni.fi.wrapper.FormPairWrapper;
import cz.muni.fi.wrapper.ListFormPairWrapper;
import cz.muni.fi.wrapper.SessionMessageWrapper;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.bind.support.SessionStatus;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import java.security.Principal;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 4.3.12
 */

@Controller
@RequestMapping("/forms/massmail")
@SessionAttributes(types = ListFormPairWrapper.class)
public class MassMailSendForm {

    private final FormManager formManager;

    private final UserManager userManager;


    private final MailCommons mailCommons;


    protected final Log logger = LogFactory.getLog(getClass());


    @Autowired
    public MassMailSendForm(FormManager manager, UserManager userManager, MailCommons mailCommons) {
        this.formManager = manager;
        this.userManager = userManager;
        this.mailCommons = mailCommons;
    }

    @InitBinder
    public void setAllowedFields(WebDataBinder dataBinder) {
        dataBinder.registerCustomEditor(FormPair.class, new FormPairEditor(formManager));
    }

    @RequestMapping(method = RequestMethod.GET)
    public String setupForm(HttpSession session, Model model) {
        logger.debug("Rendering mass mail send pair form.");
        session.removeAttribute("sessionMessage");
        Period period = (Period) session.getAttribute("sessionPeriod");
        List<FormPair> pairs;
        if (period == null) {
            pairs = this.formManager.getAllFormPairs();
        } else {
            pairs = this.formManager.getAllFormPairs(period);
        }
        List<FormPair> pairsFin = new ArrayList<FormPair>();
        for (FormPair pair : pairs) {
            if (pair.getMailTimestamp() == null) {
                pairsFin.add(pair);
            }
        }
        ListFormPairWrapper wrapper = new ListFormPairWrapper();
        wrapper.setFormPairs(pairsFin);
        model.addAttribute("listFormPairWrapper", wrapper);
        return "forms/massmail";
    }

    @RequestMapping(method = RequestMethod.POST)
    public String processSubmit(@ModelAttribute ListFormPairWrapper listFormPairWrapper,
                                BindingResult result, SessionStatus status, HttpSession session, Principal principal, HttpServletRequest request) {
        logger.debug("Processing mass mail send form.");
        session.removeAttribute("sessionMessage");

        new ListFormPairValidator().validate(listFormPairWrapper, result);
        if (result.hasErrors()) {
            logger.debug("Form validation failed");
            return "forms/massmail";
        } else {
            int mailSent = 0;
            String myId = principal.getName();
            User me = userManager.getById(myId);
            String myName = me.getName();
            String myMail = me.getEmail();
            try {
                mailCommons.sendMailsAboutForm(listFormPairWrapper.getFormPairs(), request, myName, myMail);
                for (FormPair pair : listFormPairWrapper.getFormPairs()) {
                    formManager.setFormPairTimestamp(pair);
                }
                logger.info("Mails were successfully sent.");
            } catch (EmailSendException e) {
                logger.error("Error while sending mail, mails were not sent.", e);
                session.setAttribute("sessionMessage", new SessionMessageWrapper("errorMessage", "mail.error"));
            }
            logger.debug("Mails wer sent successfully.");
            if (session.getAttribute("sessionMessage") == null) {
                SessionMessageWrapper message = new SessionMessageWrapper("okMessage", "mail.was.sent");
                session.setAttribute("sessionMessage", message);
            }
            status.setComplete();
            return "redirect:/forms/list";
        }
    }
}
