package cz.muni.fi.web;

import cz.muni.fi.domain.Form;
import cz.muni.fi.domain.FormPair;
import cz.muni.fi.domain.Participant;
import cz.muni.fi.exception.DataIntegrityViolationException;
import cz.muni.fi.service.FormManager;
import cz.muni.fi.validation.FormValidator;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.bind.support.SessionStatus;

/**
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 14.8.11
 */

@Controller
@RequestMapping("/public/forms/{formId}")
@SessionAttributes(types = {Form.class, Participant.class})
public class SaveRatingForm {
    private final FormManager formManager;

    protected final Log logger = LogFactory.getLog(getClass());

    @Autowired
    public SaveRatingForm(FormManager manager) {
        this.formManager = manager;
    }

    @InitBinder
    public void setAllowedFields(WebDataBinder dataBinder) {
        dataBinder.setDisallowedFields("id");
    }

    @RequestMapping(method = RequestMethod.GET)
    public String setupForm(@PathVariable("formId") String formHash, Model model) {
        logger.debug("Rendering public form form at " + formHash);
        Form form = this.formManager.getFormByHash(formHash);
        if (form == null) {
            logger.debug("Form " + formHash + " was not found.");
            return "redirect:/public/forms/notfound";
        }
        try {
            if (formManager.canBeAnswered(form)) {
                logger.debug("Form can be answered.");
                if ("new".equals(form.getState())) {
                    formManager.openForm(form);
                }
            } else {
                logger.debug("Form can't be answered anymore.");
                return "redirect:/public/forms/timeout";
            }

        } catch (DataIntegrityViolationException e) {
            logger.error("Form is in corrupted state", e);
            return "error";
        }
        FormPair pair = formManager.findFormPair(form);
        Participant other;
        if ("student".equals(form.getOwner().getRole())) {
            other = pair.getManagersForm().getOwner();
        } else {
            other = pair.getStudentsForm().getOwner();
        }
        model.addAttribute("other",other);
        model.addAttribute("form", form);
        return "forms/rate";
    }

    @RequestMapping(method = RequestMethod.PUT)
    public String processSubmit(@ModelAttribute Form form, BindingResult result, SessionStatus status) {
        logger.debug("Saving form.");
        new FormValidator().validate(form, result);
        if (result.hasErrors()) {
            logger.debug("There are validation errors.");
            return "forms/rate";
        } else {
            this.formManager.saveForm(form);
            status.setComplete();
            return "redirect:/public/forms/thanks";
        }
    }
}
