package cz.muni.fi.test;

import cz.muni.fi.dao.FeatureRateDao;
import cz.muni.fi.domain.Feature;
import cz.muni.fi.domain.FeatureRate;
import cz.muni.fi.domain.Organization;
import cz.muni.fi.domain.Period;
import cz.muni.fi.service.*;
import org.junit.After;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import java.util.List;

import static junit.framework.Assert.assertNull;
import static org.junit.Assert.assertEquals;


/**
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 29.1.12
 */
@RunWith(SpringJUnit4ClassRunner.class)
@ContextConfiguration(locations = {"/applicationContext-test.xml"})
public class AggregateFeatureRateManagerTest {
    @After
    public void tearDown() {
        assertEquals(0, featureManager.getAllFeatures().size());
        assertEquals(0,organizationManager.getAllOrganizations().size());
    }

    @Autowired
    private AggregatedFeatureRateManager aggregatedFeatureRateManager;

    @Autowired
    private FeatureManager featureManager;

    @Autowired
    private FormManager formManager;

    @Autowired
    private ParticipantManager participantManager;

    @Autowired
    private OrganizationManager organizationManager;

    @Autowired
    private PeriodManager periodManager;

    @Autowired
    private FeatureRateDao featureRateDao;

    @Test
    public void testGetRateForFeature() throws Exception {
        Period period = createSamplePeriod();
        periodManager.addPeriod(period);
        Feature feature = createSampleFeature();
        featureManager.addFeature(feature);
        Feature feature2 = createSampleFeature();
        feature2.setParent(feature);
        featureManager.addFeature(feature2);
        Feature feature3 = createSampleFeature();
        feature3.setParent(feature);
        featureManager.addFeature(feature3);
        feature = featureManager.getFeatureById(feature.getId());

        //global
        FeatureRate result = aggregatedFeatureRateManager.getRateForFeature(feature);
        assertEquals(result.getFeature(), feature);
        assertEquals(0, result.getResponses());
        assertEquals(0, result.getSum());
        assertEquals(0, result.getValue());

        //no organization, no data
        result = aggregatedFeatureRateManager.getRateForFeature(feature, period);
        assertNull(result);

        Organization org = new Organization();
        org.setName("org");
        org.setIdNumber("3");
        organizationManager.addOrganization(org);

        //per period
        result = aggregatedFeatureRateManager.getRateForFeature(feature, period);
        assertEquals(result.getPeriod(), period);
        assertEquals(result.getFeature(), feature);
        assertEquals(0, result.getResponses());
        assertEquals(0, result.getSum());
        assertEquals(0, result.getValue());

        FeatureRate rate = featureRateDao.findByFeature(feature, period, org);
        rate.setSum(24);
        rate.setResponses(3);
        featureRateDao.update(rate);
        rate = featureRateDao.findByFeature(feature2, period, org);
        rate.setSum(10);
        rate.setResponses(1);
        featureRateDao.update(rate);
        rate = featureRateDao.findByFeature(feature3, period, org);
        rate.setSum(100);
        rate.setResponses(2);
        featureRateDao.update(rate);
        result = aggregatedFeatureRateManager.getRateForFeature(feature, period, org);
        assertEquals(6, result.getResponses());
        assertEquals(134, result.getSum());
        assertEquals(22, result.getValue());

        result = aggregatedFeatureRateManager.getRateForFeature(feature, org);
        assertEquals(6, result.getResponses());
        assertEquals(134, result.getSum());
        assertEquals(22, result.getValue());

        result = aggregatedFeatureRateManager.getRateForFeature(feature, period);
        assertEquals(6, result.getResponses());
        assertEquals(134, result.getSum());
        assertEquals(22, result.getValue());

        result = aggregatedFeatureRateManager.getRateForFeature(feature);
        assertEquals(6, result.getResponses());
        assertEquals(134, result.getSum());
        assertEquals(22, result.getValue());

        result = aggregatedFeatureRateManager.getRateForFeature(feature2, period, org);
        assertEquals(1, result.getResponses());
        assertEquals(10, result.getSum());
        assertEquals(10, result.getValue());

        result = aggregatedFeatureRateManager.getRateForFeature(feature2, org);
        assertEquals(1, result.getResponses());
        assertEquals(10, result.getSum());
        assertEquals(10, result.getValue());

        result = aggregatedFeatureRateManager.getRateForFeature(feature2, period);
        assertEquals(1, result.getResponses());
        assertEquals(10, result.getSum());
        assertEquals(10, result.getValue());

        result = aggregatedFeatureRateManager.getRateForFeature(feature2);
        assertEquals(1, result.getResponses());
        assertEquals(10, result.getSum());
        assertEquals(10, result.getValue());

        Feature feature4 = createSampleFeature();
        featureManager.addFeature(feature4);
        feature.setParent(feature4);
        featureManager.updateFeature(feature);
        rate = featureRateDao.findByFeature(feature4, period, org);
        rate.setSum(10);
        rate.setResponses(5);
        featureRateDao.update(rate);

        result = aggregatedFeatureRateManager.getRateForFeature(feature4, period, org);
        assertEquals(11, result.getResponses());
        assertEquals(144, result.getSum());
        assertEquals(13, result.getValue());

        result = aggregatedFeatureRateManager.getRateForFeature(feature4, org);
        assertEquals(11, result.getResponses());
        assertEquals(144, result.getSum());
        assertEquals(13, result.getValue());

        result = aggregatedFeatureRateManager.getRateForFeature(feature4, period);
        assertEquals(11, result.getResponses());
        assertEquals(144, result.getSum());
        assertEquals(13, result.getValue());

        result = aggregatedFeatureRateManager.getRateForFeature(feature4);
        assertEquals(11, result.getResponses());
        assertEquals(144, result.getSum());
        assertEquals(13, result.getValue());

        featureManager.deleteFeature(feature4);
        organizationManager.deleteOrganization(org);
        periodManager.deletePeriod(period);
    }

    @Test
    public void testGetAverage() throws Exception {
        Period period = createSamplePeriod();
        Period period2 = createSamplePeriod();
        Organization org = new Organization();
        org.setName("org");
        org.setIdNumber("344");
        organizationManager.addOrganization(org);
        Organization org2 = new Organization();
        org2.setName("org2");
        org2.setIdNumber("3443");
        organizationManager.addOrganization(org2);
        periodManager.addPeriod(period);
        periodManager.addPeriod(period2);
        Feature feature = createSampleFeature();
        Feature feature2 = createSampleFeature();
        Feature feature3 = createSampleFeature();
        Feature feature4 = createSampleFeature();
        Feature feature5 = createSampleFeature();
        Feature feature6 = createSampleFeature();
        featureManager.addFeature(feature);
        featureManager.addFeature(feature3);
        feature2.setParent(feature);
        featureManager.addFeature(feature2);
        feature4.setParent(feature3);
        feature5.setParent(feature3);
        featureManager.addFeature(feature4);
        featureManager.addFeature(feature5);
        feature6.setParent(feature4);
        featureManager.addFeature(feature6);

        FeatureRate rate = featureRateDao.findByFeature(feature, period, org);
        rate.setSum(5);
        rate.setResponses(1);
        featureRateDao.update(rate);

        rate = featureRateDao.findByFeature(feature2, period, org);
        rate.setSum(6);
        rate.setResponses(2);
        featureRateDao.update(rate);

        rate = featureRateDao.findByFeature(feature, period2, org);
        rate.setSum(10);
        rate.setResponses(1);
        featureRateDao.update(rate);

        rate = featureRateDao.findByFeature(feature2, period2, org);
        rate.setSum(8);
        rate.setResponses(2);
        featureRateDao.update(rate);

        rate = featureRateDao.findByFeature(feature, period, org2);
        rate.setSum(15);
        rate.setResponses(1);
        featureRateDao.update(rate);

        rate = featureRateDao.findByFeature(feature2, period, org2);
        rate.setSum(10);
        rate.setResponses(2);
        featureRateDao.update(rate);

        rate = featureRateDao.findByFeature(feature, period2, org2);
        rate.setSum(20);
        rate.setResponses(1);
        featureRateDao.update(rate);

        rate = featureRateDao.findByFeature(feature2, period2, org2);
        rate.setSum(20);
        rate.setResponses(2);
        featureRateDao.update(rate);
        int result = aggregatedFeatureRateManager.getAverage();
        assertEquals(8, result);
        result = aggregatedFeatureRateManager.getAverage(null, null);
        assertEquals(8, result);
        result = aggregatedFeatureRateManager.getAverage((Organization) null);
        assertEquals(8, result);
        result = aggregatedFeatureRateManager.getAverage((Period) null);
        assertEquals(8, result);
        result = aggregatedFeatureRateManager.getAverage(period);
        assertEquals(6, result);
        result = aggregatedFeatureRateManager.getAverage(period2);
        assertEquals(10, result);
        result = aggregatedFeatureRateManager.getAverage(org);
        assertEquals(5, result);
        result = aggregatedFeatureRateManager.getAverage(org2);
        assertEquals(11, result);

        result = aggregatedFeatureRateManager.getAverage(period, null);
        assertEquals(6, result);
        result = aggregatedFeatureRateManager.getAverage(period2, null);
        assertEquals(10, result);
        result = aggregatedFeatureRateManager.getAverage(null, org);
        assertEquals(5, result);
        result = aggregatedFeatureRateManager.getAverage(null, org2);
        assertEquals(11, result);

        result = aggregatedFeatureRateManager.getAverage(period, org);
        assertEquals(4, result);
        result = aggregatedFeatureRateManager.getAverage(period2, org);
        assertEquals(6, result);
        result = aggregatedFeatureRateManager.getAverage(period, org2);
        assertEquals(8, result);
        result = aggregatedFeatureRateManager.getAverage(period2, org2);
        assertEquals(13, result);

        rate = featureRateDao.findByFeature(feature3, period, org);
        rate.setSum(2);
        rate.setResponses(1);
        featureRateDao.update(rate);

        rate = featureRateDao.findByFeature(feature3, period2, org);
        rate.setSum(6);
        rate.setResponses(2);
        featureRateDao.update(rate);

        rate = featureRateDao.findByFeature(feature3, period, org2);
        rate.setSum(8);
        rate.setResponses(2);
        featureRateDao.update(rate);

        rate = featureRateDao.findByFeature(feature3, period2, org2);
        rate.setSum(10);
        rate.setResponses(1);
        featureRateDao.update(rate);

        result = aggregatedFeatureRateManager.getAverage();
        assertEquals(6, result);
        result = aggregatedFeatureRateManager.getAverage(null, null);
        assertEquals(6, result);
        result = aggregatedFeatureRateManager.getAverage((Organization) null);
        assertEquals(6, result);
        result = aggregatedFeatureRateManager.getAverage((Period) null);
        assertEquals(6, result);
        result = aggregatedFeatureRateManager.getAverage(period);
        assertEquals(5, result);
        result = aggregatedFeatureRateManager.getAverage(period2);
        assertEquals(8, result);
        result = aggregatedFeatureRateManager.getAverage(org);
        assertEquals(4, result);
        result = aggregatedFeatureRateManager.getAverage(org2);
        assertEquals(9, result);

        result = aggregatedFeatureRateManager.getAverage(period, null);
        assertEquals(5, result);
        result = aggregatedFeatureRateManager.getAverage(period2, null);
        assertEquals(8, result);
        result = aggregatedFeatureRateManager.getAverage(null, org);
        assertEquals(4, result);
        result = aggregatedFeatureRateManager.getAverage(null, org2);
        assertEquals(9, result);

        result = aggregatedFeatureRateManager.getAverage(period, org);
        assertEquals(3, result);
        result = aggregatedFeatureRateManager.getAverage(period2, org);
        assertEquals(5, result);
        result = aggregatedFeatureRateManager.getAverage(period, org2);
        assertEquals(6, result);
        result = aggregatedFeatureRateManager.getAverage(period2, org2);
        assertEquals(12, result);


        featureManager.deleteFeature(feature);
        featureManager.deleteFeature(feature3);
        periodManager.deletePeriod(period);
        periodManager.deletePeriod(period2);
        organizationManager.deleteOrganization(org);
        organizationManager.deleteOrganization(org2);
    }

    @Test
    public void testGetTotalImprovementCount() throws Exception {
         Period period = createSamplePeriod();
        Period period2 = createSamplePeriod();
        Organization org = new Organization();
        org.setName("org");
        org.setIdNumber("344");
        organizationManager.addOrganization(org);
        Organization org2 = new Organization();
        org2.setName("org2");
        org2.setIdNumber("3443");
        organizationManager.addOrganization(org2);
        periodManager.addPeriod(period);
        periodManager.addPeriod(period2);
        Feature feature = createSampleFeature();
        Feature feature2 = createSampleFeature();
        Feature feature3 = createSampleFeature();
        Feature feature4 = createSampleFeature();
        Feature feature5 = createSampleFeature();
        Feature feature6 = createSampleFeature();
        featureManager.addFeature(feature);
        featureManager.addFeature(feature3);
        feature2.setParent(feature);
        featureManager.addFeature(feature2);
        feature4.setParent(feature3);
        feature5.setParent(feature3);
        featureManager.addFeature(feature4);
        featureManager.addFeature(feature5);
        feature6.setParent(feature4);
        featureManager.addFeature(feature6);

        FeatureRate rate = featureRateDao.findByFeature(feature, period, org);
        rate.setSum(5);
        rate.setResponses(1);
        featureRateDao.update(rate);

        rate = featureRateDao.findByFeature(feature2, period, org);
        rate.setSum(6);
        rate.setResponses(2);
        featureRateDao.update(rate);

        rate = featureRateDao.findByFeature(feature, period2, org);
        rate.setSum(10);
        rate.setResponses(1);
        featureRateDao.update(rate);

        rate = featureRateDao.findByFeature(feature2, period2, org);
        rate.setSum(8);
        rate.setResponses(2);
        featureRateDao.update(rate);

        rate = featureRateDao.findByFeature(feature, period, org2);
        rate.setSum(15);
        rate.setResponses(1);
        featureRateDao.update(rate);

        rate = featureRateDao.findByFeature(feature2, period, org2);
        rate.setSum(10);
        rate.setResponses(2);
        featureRateDao.update(rate);

        rate = featureRateDao.findByFeature(feature, period2, org2);
        rate.setSum(20);
        rate.setResponses(1);
        featureRateDao.update(rate);

        rate = featureRateDao.findByFeature(feature2, period2, org2);
        rate.setSum(20);
        rate.setResponses(2);
        featureRateDao.update(rate);
        int result = aggregatedFeatureRateManager.getAverage();
        assertEquals(8, result);
        result = aggregatedFeatureRateManager.getAverage(null, null);
        assertEquals(8, result);
        result = aggregatedFeatureRateManager.getAverage((Organization) null);
        assertEquals(8, result);
        result = aggregatedFeatureRateManager.getAverage((Period) null);
        assertEquals(8, result);
        result = aggregatedFeatureRateManager.getAverage(period);
        assertEquals(6, result);
        result = aggregatedFeatureRateManager.getAverage(period2);
        assertEquals(10, result);
        result = aggregatedFeatureRateManager.getAverage(org);
        assertEquals(5, result);
        result = aggregatedFeatureRateManager.getAverage(org2);
        assertEquals(11, result);

        result = aggregatedFeatureRateManager.getAverage(period, null);
        assertEquals(6, result);
        result = aggregatedFeatureRateManager.getAverage(period2, null);
        assertEquals(10, result);
        result = aggregatedFeatureRateManager.getAverage(null, org);
        assertEquals(5, result);
        result = aggregatedFeatureRateManager.getAverage(null, org2);
        assertEquals(11, result);

        result = aggregatedFeatureRateManager.getAverage(period, org);
        assertEquals(4, result);
        result = aggregatedFeatureRateManager.getAverage(period2, org);
        assertEquals(6, result);
        result = aggregatedFeatureRateManager.getAverage(period, org2);
        assertEquals(8, result);
        result = aggregatedFeatureRateManager.getAverage(period2, org2);
        assertEquals(13, result);

        //improvement count
         result = aggregatedFeatureRateManager.getTotalImprovementCount(true);
        assertEquals(0, result);
        result = aggregatedFeatureRateManager.getTotalImprovementCount(true,null, null);
        assertEquals(0, result);
        result = aggregatedFeatureRateManager.getTotalImprovementCount(true,(Organization) null);
        assertEquals(0, result);
        result = aggregatedFeatureRateManager.getTotalImprovementCount(true,(Period) null);
        assertEquals(0, result);
        result = aggregatedFeatureRateManager.getTotalImprovementCount(true,period);
        assertEquals(0, result);
        result = aggregatedFeatureRateManager.getTotalImprovementCount(true,period2);
        assertEquals(0, result);
        result = aggregatedFeatureRateManager.getTotalImprovementCount(true,org);
        assertEquals(0, result);
        result = aggregatedFeatureRateManager.getTotalImprovementCount(true,org2);
        assertEquals(0, result);

        result = aggregatedFeatureRateManager.getTotalImprovementCount(true,period, null);
        assertEquals(0, result);
        result = aggregatedFeatureRateManager.getTotalImprovementCount(true,period2, null);
        assertEquals(0, result);
        result = aggregatedFeatureRateManager.getTotalImprovementCount(true,null, org);
        assertEquals(0, result);
        result = aggregatedFeatureRateManager.getTotalImprovementCount(true,null, org2);
        assertEquals(0, result);

        result = aggregatedFeatureRateManager.getTotalImprovementCount(true,period, org);
        assertEquals(0, result);
        result = aggregatedFeatureRateManager.getTotalImprovementCount(true,period2, org);
        assertEquals(0, result);
        result = aggregatedFeatureRateManager.getTotalImprovementCount(true,period, org2);
        assertEquals(0, result);
        result = aggregatedFeatureRateManager.getTotalImprovementCount(true,period2, org2);
        assertEquals(0, result);
        
        rate = featureRateDao.findByFeature(feature3, period, org);
        rate.setSum(2);
        rate.setResponses(1);
        featureRateDao.update(rate);

        rate = featureRateDao.findByFeature(feature3, period2, org);
        rate.setSum(6);
        rate.setResponses(2);
        featureRateDao.update(rate);

        rate = featureRateDao.findByFeature(feature3, period, org2);
        rate.setSum(8);
        rate.setResponses(2);
        featureRateDao.update(rate);

        rate = featureRateDao.findByFeature(feature3, period2, org2);
        rate.setSum(10);
        rate.setResponses(1);
        featureRateDao.update(rate);

        result = aggregatedFeatureRateManager.getAverage();
        assertEquals(6, result);
        result = aggregatedFeatureRateManager.getAverage(null, null);
        assertEquals(6, result);
        result = aggregatedFeatureRateManager.getAverage((Organization) null);
        assertEquals(6, result);
        result = aggregatedFeatureRateManager.getAverage((Period) null);
        assertEquals(6, result);
        result = aggregatedFeatureRateManager.getAverage(period);
        assertEquals(5, result);
        result = aggregatedFeatureRateManager.getAverage(period2);
        assertEquals(8, result);
        result = aggregatedFeatureRateManager.getAverage(org);
        assertEquals(4, result);
        result = aggregatedFeatureRateManager.getAverage(org2);
        assertEquals(9, result);

        result = aggregatedFeatureRateManager.getAverage(period, null);
        assertEquals(5, result);
        result = aggregatedFeatureRateManager.getAverage(period2, null);
        assertEquals(8, result);
        result = aggregatedFeatureRateManager.getAverage(null, org);
        assertEquals(4, result);
        result = aggregatedFeatureRateManager.getAverage(null, org2);
        assertEquals(9, result);

        result = aggregatedFeatureRateManager.getAverage(period, org);
        assertEquals(3, result);
        result = aggregatedFeatureRateManager.getAverage(period2, org);
        assertEquals(5, result);
        result = aggregatedFeatureRateManager.getAverage(period, org2);
        assertEquals(6, result);
        result = aggregatedFeatureRateManager.getAverage(period2, org2);
        assertEquals(12, result);

         //improvement count
         result = aggregatedFeatureRateManager.getTotalImprovementCount(true);
        assertEquals(2, result);
        result = aggregatedFeatureRateManager.getTotalImprovementCount(true,null, null);
        assertEquals(2, result);
        result = aggregatedFeatureRateManager.getTotalImprovementCount(true,(Organization) null);
        assertEquals(2, result);
        result = aggregatedFeatureRateManager.getTotalImprovementCount(true,(Period) null);
        assertEquals(2, result);
        result = aggregatedFeatureRateManager.getTotalImprovementCount(true,period);
        assertEquals(2, result);
        result = aggregatedFeatureRateManager.getTotalImprovementCount(true,period2);
        assertEquals(3, result);
        result = aggregatedFeatureRateManager.getTotalImprovementCount(true,org);
        assertEquals(1, result);
        result = aggregatedFeatureRateManager.getTotalImprovementCount(true,org2);
        assertEquals(3, result);

        result = aggregatedFeatureRateManager.getTotalImprovementCount(true,period, null);
        assertEquals(2, result);
        result = aggregatedFeatureRateManager.getTotalImprovementCount(true,period2, null);
        assertEquals(3, result);
        result = aggregatedFeatureRateManager.getTotalImprovementCount(true,null, org);
        assertEquals(1, result);
        result = aggregatedFeatureRateManager.getTotalImprovementCount(true,null, org2);
        assertEquals(3, result);

        result = aggregatedFeatureRateManager.getTotalImprovementCount(true,period, org);
        assertEquals(1, result);
        result = aggregatedFeatureRateManager.getTotalImprovementCount(true,period2, org);
        assertEquals(2, result);
        result = aggregatedFeatureRateManager.getTotalImprovementCount(true,period, org2);
        assertEquals(2, result);
        result = aggregatedFeatureRateManager.getTotalImprovementCount(true,period2, org2);
        assertEquals(2, result);


        featureManager.deleteFeature(feature);
        featureManager.deleteFeature(feature3);
        periodManager.deletePeriod(period);
        periodManager.deletePeriod(period2);
        organizationManager.deleteOrganization(org);
        organizationManager.deleteOrganization(org2);

    }


    @Test
    public void testGetAggregatedFeatureRates() throws Exception{
        Period period = createSamplePeriod();
        Period period2 = createSamplePeriod();
        Organization org = new Organization();
        org.setName("org");
        org.setIdNumber("344");
        organizationManager.addOrganization(org);
        Organization org2 = new Organization();
        org2.setName("org2");
        org2.setIdNumber("3443");
        organizationManager.addOrganization(org2);
        periodManager.addPeriod(period);
        periodManager.addPeriod(period2);
        Feature feature = createSampleFeature();
        Feature feature2 = createSampleFeature();
        Feature feature3 = createSampleFeature();
        Feature feature4 = createSampleFeature();
        Feature feature5 = createSampleFeature();
        Feature feature6 = createSampleFeature();
        featureManager.addFeature(feature);
        featureManager.addFeature(feature3);
        feature2.setParent(feature);
        featureManager.addFeature(feature2);
        feature4.setParent(feature3);
        feature5.setParent(feature3);
        featureManager.addFeature(feature4);
        featureManager.addFeature(feature5);
        feature6.setParent(feature4);
        featureManager.addFeature(feature6);

        FeatureRate rate = featureRateDao.findByFeature(feature, period, org);
        rate.setSum(5);
        rate.setResponses(1);
        featureRateDao.update(rate);

        rate = featureRateDao.findByFeature(feature2, period, org);
        rate.setSum(6);
        rate.setResponses(2);
        featureRateDao.update(rate);

        rate = featureRateDao.findByFeature(feature, period2, org);
        rate.setSum(10);
        rate.setResponses(1);
        featureRateDao.update(rate);

        rate = featureRateDao.findByFeature(feature2, period2, org);
        rate.setSum(8);
        rate.setResponses(2);
        featureRateDao.update(rate);

        rate = featureRateDao.findByFeature(feature, period, org2);
        rate.setSum(15);
        rate.setResponses(1);
        featureRateDao.update(rate);

        rate = featureRateDao.findByFeature(feature2, period, org2);
        rate.setSum(10);
        rate.setResponses(2);
        featureRateDao.update(rate);

        rate = featureRateDao.findByFeature(feature, period2, org2);
        rate.setSum(20);
        rate.setResponses(1);
        featureRateDao.update(rate);

        rate = featureRateDao.findByFeature(feature2, period2, org2);
        rate.setSum(20);
        rate.setResponses(2);
        featureRateDao.update(rate);

        List<FeatureRate> result = aggregatedFeatureRateManager.getAggregatedFeatureRates(org);
        assertEquals(2, result.size());
        assertEquals(6, result.get(0).getResponses());
        assertEquals(29, result.get(0).getSum());
        assertEquals(0, result.get(1).getResponses());
        assertEquals(0, result.get(1).getSum());

        result = aggregatedFeatureRateManager.getAggregatedFeatureRates(org2);
        assertEquals(2, result.size());
        assertEquals(6, result.get(0).getResponses());
        assertEquals(65, result.get(0).getSum());
        assertEquals(0, result.get(1).getResponses());
        assertEquals(0, result.get(1).getSum());

        result = aggregatedFeatureRateManager.getAggregatedFeatureRates(period);
        assertEquals(2, result.size());
        assertEquals(6, result.get(0).getResponses());
        assertEquals(36, result.get(0).getSum());
        assertEquals(0, result.get(1).getResponses());
        assertEquals(0, result.get(1).getSum());

        result = aggregatedFeatureRateManager.getAggregatedFeatureRates(period2);
        assertEquals(2, result.size());
        assertEquals(6, result.get(0).getResponses());
        assertEquals(58, result.get(0).getSum());
        assertEquals(0, result.get(1).getResponses());
        assertEquals(0, result.get(1).getSum());

        result = aggregatedFeatureRateManager.getAggregatedFeatureRates();
        assertEquals(2, result.size());
        assertEquals(12, result.get(0).getResponses());
        assertEquals(94, result.get(0).getSum());
        assertEquals(0, result.get(1).getResponses());
        assertEquals(0, result.get(1).getSum());

        rate = featureRateDao.findByFeature(feature3, period, org);
        rate.setSum(2);
        rate.setResponses(1);
        featureRateDao.update(rate);

        rate = featureRateDao.findByFeature(feature3, period2, org);
        rate.setSum(6);
        rate.setResponses(2);
        featureRateDao.update(rate);

        rate = featureRateDao.findByFeature(feature3, period, org2);
        rate.setSum(8);
        rate.setResponses(2);
        featureRateDao.update(rate);

        rate = featureRateDao.findByFeature(feature3, period2, org2);
        rate.setSum(10);
        rate.setResponses(1);
        featureRateDao.update(rate);

        result = aggregatedFeatureRateManager.getAggregatedFeatureRates(org);
        assertEquals(2, result.size());
        assertEquals(6, result.get(0).getResponses());
        assertEquals(29, result.get(0).getSum());
        assertEquals(3, result.get(1).getResponses());
        assertEquals(8, result.get(1).getSum());

        result = aggregatedFeatureRateManager.getAggregatedFeatureRates(org2);
        assertEquals(2, result.size());
        assertEquals(6, result.get(0).getResponses());
        assertEquals(65, result.get(0).getSum());
        assertEquals(3, result.get(1).getResponses());
        assertEquals(18, result.get(1).getSum());

        result = aggregatedFeatureRateManager.getAllFeatureRates(null, org2);
        assertEquals(2, result.size());
        assertEquals(6, result.get(0).getResponses());
        assertEquals(65, result.get(0).getSum());
        assertEquals(3, result.get(1).getResponses());
        assertEquals(18, result.get(1).getSum());

        result = aggregatedFeatureRateManager.getAggregatedFeatureRates(period);
        assertEquals(2, result.size());
        assertEquals(6, result.get(0).getResponses());
        assertEquals(36, result.get(0).getSum());
        assertEquals(3, result.get(1).getResponses());
        assertEquals(10, result.get(1).getSum());

        result = aggregatedFeatureRateManager.getAggregatedFeatureRates(period2);
        assertEquals(2, result.size());
        assertEquals(6, result.get(0).getResponses());
        assertEquals(58, result.get(0).getSum());
        assertEquals(3, result.get(1).getResponses());
        assertEquals(16, result.get(1).getSum());

        result = aggregatedFeatureRateManager.getAllFeatureRates(period2, null);
        assertEquals(2, result.size());
        assertEquals(6, result.get(0).getResponses());
        assertEquals(58, result.get(0).getSum());
        assertEquals(3, result.get(1).getResponses());
        assertEquals(16, result.get(1).getSum());

        result = aggregatedFeatureRateManager.getAggregatedFeatureRates();
        assertEquals(2, result.size());
        assertEquals(12, result.get(0).getResponses());
        assertEquals(94, result.get(0).getSum());
        assertEquals(6, result.get(1).getResponses());
        assertEquals(26, result.get(1).getSum());

        featureManager.deleteFeature(feature);
        featureManager.deleteFeature(feature3);
        periodManager.deletePeriod(period);
        periodManager.deletePeriod(period2);
        organizationManager.deleteOrganization(org);
        organizationManager.deleteOrganization(org2);
    }

    @Test
    public void getAllFeaturesTest() throws Exception {
        Feature feature = createSampleFeature();
        Feature feature2 = createSampleFeature();
        Feature feature3 = createSampleFeature();
        Feature feature4 = createSampleFeature();
        Feature feature5 = createSampleFeature();
        Feature feature6 = createSampleFeature();
        featureManager.addFeature(feature);
        featureManager.addFeature(feature3);
        feature2.setParent(feature);
        featureManager.addFeature(feature2);
        feature4.setParent(feature3);
        feature5.setParent(feature3);
        featureManager.addFeature(feature4);
        featureManager.addFeature(feature5);
        feature6.setParent(feature4);
        featureManager.addFeature(feature6);
        Period period = createSamplePeriod();
        Organization org = new Organization();
        org.setName("org");
        org.setIdNumber("344");
        organizationManager.addOrganization(org);
        periodManager.addPeriod(period);
        FeatureRate rate = featureRateDao.findByFeature(feature, period, org);
        rate.setSum(5);
        rate.setResponses(1);
        featureRateDao.update(rate);

        rate = featureRateDao.findByFeature(feature2, period, org);
        rate.setSum(20);
        rate.setResponses(2);
        featureRateDao.update(rate);

        rate = featureRateDao.findByFeature(feature3, period, org);
        rate.setSum(8);
        rate.setResponses(4);
        featureRateDao.update(rate);

        rate = featureRateDao.findByFeature(feature4, period, org);
        rate.setSum(10);
        rate.setResponses(1);
        featureRateDao.update(rate);

        rate = featureRateDao.findByFeature(feature5, period, org);
        rate.setSum(5);
        rate.setResponses(5);
        featureRateDao.update(rate);

        rate = featureRateDao.findByFeature(feature6, period, org);
        rate.setSum(100);
        rate.setResponses(2);
        featureRateDao.update(rate);

        List<FeatureRate> result = aggregatedFeatureRateManager.getAllFeatureRates(period, org);
        assertEquals(2, result.size());
        assertEquals(25, result.get(0).getSum());
        assertEquals(3, result.get(0).getResponses());
        assertEquals(8, result.get(0).getValue());

        assertEquals(123, result.get(1).getSum());
        assertEquals(12, result.get(1).getResponses());
        assertEquals(10, result.get(1).getValue());

        feature4.setParent(null);
        featureManager.updateFeature(feature4);
        result = aggregatedFeatureRateManager.getAllFeatureRates(period, org);
        assertEquals(3, result.size());
        assertEquals(25, result.get(0).getSum());
        assertEquals(3, result.get(0).getResponses());
        assertEquals(8, result.get(0).getValue());

        assertEquals(13, result.get(1).getSum());
        assertEquals(9, result.get(1).getResponses());
        assertEquals(1, result.get(1).getValue());

        assertEquals(110, result.get(2).getSum());
        assertEquals(3, result.get(2).getResponses());
        assertEquals(37, result.get(2).getValue());

        Period period2 = createSamplePeriod();
        periodManager.addPeriod(period2);

        rate = featureRateDao.findByFeature(feature2, period2, org);
        rate.setSum(20);
        rate.setResponses(2);
        featureRateDao.update(rate);

        rate = featureRateDao.findByFeature(feature3, period2, org);
        rate.setSum(8);
        rate.setResponses(4);
        featureRateDao.update(rate);

        result = aggregatedFeatureRateManager.getAllFeatureRates(period, org);
        assertEquals(3, result.size());
        assertEquals(25, result.get(0).getSum());
        assertEquals(3, result.get(0).getResponses());
        assertEquals(8, result.get(0).getValue());

        assertEquals(13, result.get(1).getSum());
        assertEquals(9, result.get(1).getResponses());
        assertEquals(1, result.get(1).getValue());

        assertEquals(110, result.get(2).getSum());
        assertEquals(3, result.get(2).getResponses());
        assertEquals(37, result.get(2).getValue());

        featureManager.deleteFeature(feature);
        featureManager.deleteFeature(feature4);
        featureManager.deleteFeature(feature3);
        periodManager.deletePeriod(period);
        periodManager.deletePeriod(period2);
        organizationManager.deleteOrganization(org);
    }

    private static Feature createSampleFeature() {
        Feature feature = new Feature();
        feature.setHint("hint");
        feature.setManagerQuestion("manager");
        feature.setStudentQuestion("student");
        feature.setName("name");
        return feature;
    }

    private static Period createSamplePeriod() {
        Period period = new Period();
        period.setDescription("decription");
        period.setNote("note");
        period.setYear(2011);
        return period;
    }
}

