package cz.muni.fi.test;

import cz.muni.fi.dao.FormDao;
import cz.muni.fi.dao.FormPairDao;
import cz.muni.fi.dao.PeriodDao;
import cz.muni.fi.domain.Form;
import cz.muni.fi.domain.FormPair;
import cz.muni.fi.domain.Period;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import java.util.Date;
import java.util.List;

import static junit.framework.Assert.assertNull;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;


/**
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 15.8.11
 */

@RunWith(SpringJUnit4ClassRunner.class)
// ApplicationContext will be loaded from "/applicationContext.xml" and "/applicationContext-test.xml"
// in the root of the classpath
@ContextConfiguration(locations = {"/applicationContext-test.xml"})
public class JpaFormPairDaoTest {

    @Autowired
    private FormPairDao formPairDao;

    @Autowired
    private FormDao formDao;

    @Autowired
    private PeriodDao periodDao;


    @Test
    public void testFindByForm() {
        Form form1 = new Form();
        form1.setHash("hash");
        formDao.save(form1);
        Form form2 = new Form();
        form2.setHash("hash2");
        formDao.save(form2);
        FormPair pair = new FormPair();
        pair.setState("new");
        pair.setManagersForm(form1);
        pair.setStudentsForm(form2);
        formPairDao.save(pair);
        FormPair result = formPairDao.findByForm(form1);
        assertEquals(pair, result);
        result = formPairDao.findByForm(form2);
        assertEquals(pair, result);
        formPairDao.delete(pair);
    }

    @Test
    public void testFindById() throws Exception {
        Form form1 = new Form();
        form1.setHash("hash");
        formDao.save(form1);
        Form form2 = new Form();
        form2.setHash("hash2");
        formDao.save(form2);
        FormPair pair = new FormPair();
        pair.setState("new");
        pair.setManagersForm(form1);
        pair.setStudentsForm(form2);
        formPairDao.save(pair);
        FormPair result = formPairDao.findById(pair.getId());
        assertEquals(pair, result);
        result = formPairDao.findById(14L);
        assertNull(result);
        formPairDao.delete(pair);
    }

    @Test
    public void testSave() throws Exception {
        Form form1 = new Form();
        form1.setHash("hash");
        formDao.save(form1);
        Form form2 = new Form();
        form2.setHash("hash2");
        formDao.save(form2);
        FormPair pair = new FormPair();
        pair.setState("new");
        pair.setManagersForm(form1);
        pair.setStudentsForm(form2);
        formPairDao.save(pair);
        FormPair result = formPairDao.findById(pair.getId());
        assertEquals(pair, result);
        assertEquals(pair.getManagersForm(), result.getManagersForm());
        assertEquals(pair.getStudentsForm(), result.getStudentsForm());
        assertEquals(pair.getState(), result.getState());
        assertEquals(pair.getMailTimestamp(), result.getMailTimestamp());
        formPairDao.delete(pair);
    }

    @Test
    public void testUpdate() throws Exception {
        Form form1 = new Form();
        form1.setHash("hash");
        formDao.save(form1);
        Form form2 = new Form();
        form2.setHash("hash2");
        formDao.save(form2);
        FormPair pair = new FormPair();
        pair.setState("new");
        pair.setManagersForm(form1);
        pair.setStudentsForm(form2);
        formPairDao.save(pair);
        FormPair result = formPairDao.findById(pair.getId());
        assertEquals(pair, result);
        pair.setMailTimestamp(new Date());
        pair.setState("opened");
        formPairDao.update(pair);
        result = formPairDao.findById(pair.getId());
        assertEquals(pair, result);
        assertEquals(pair.getManagersForm(), result.getManagersForm());
        assertEquals(pair.getStudentsForm(), result.getStudentsForm());
        assertEquals(pair.getState(), result.getState());
        assertEquals(pair.getMailTimestamp(), result.getMailTimestamp());
        formPairDao.delete(pair);
    }

    @Test
    public void testDelete() throws Exception {
        Form form1 = new Form();
        form1.setHash("hash");
        formDao.save(form1);
        Form form2 = new Form();
        form2.setHash("hash2");
        formDao.save(form2);
        FormPair pair = new FormPair();
        pair.setState("new");
        pair.setManagersForm(form1);
        pair.setStudentsForm(form2);
        formPairDao.save(pair);
        FormPair result = formPairDao.findById(pair.getId());
        assertEquals(pair, result);
        List<FormPair> list = formPairDao.getAllFormPairs();
        assertEquals(1, list.size());
        assertTrue(list.contains(pair));
        List<Form> forms = formDao.getAllForms();
        assertEquals(2, forms.size());
        assertTrue(forms.contains(form1));
        formPairDao.delete(pair);
        list = formPairDao.getAllFormPairs();
        assertEquals(0, list.size());
        forms = formDao.getAllForms();
        assertEquals(0, forms.size());
        result = formPairDao.findById(pair.getId());
        assertNull(result);
    }

    @Test
    public void testGetAllFormPairs() throws Exception {
        List<FormPair> pairs = formPairDao.getAllFormPairs();
        assertEquals(0, pairs.size());
        Period period = new Period();
        period.setDescription("period");
        period.setYear(2012);
        periodDao.save(period);
        pairs = formPairDao.getAllFormPairs(period);
        assertEquals(0, pairs.size());
        Form form1 = new Form();
        form1.setHash("hash");
        form1.setPeriod(period);
        formDao.save(form1);
        Form form2 = new Form();
        form2.setHash("hash2");
        form2.setPeriod(period);
        formDao.save(form2);
        FormPair pair = new FormPair();
        pair.setState("new");
        pair.setManagersForm(form1);
        pair.setStudentsForm(form2);
        formPairDao.save(pair);
        pairs = formPairDao.getAllFormPairs();
        assertEquals(1, pairs.size());
        assertTrue(pairs.contains(pair));
        Form form3 = new Form();
        form3.setHash("hash");
        formDao.save(form3);
        Form form4 = new Form();
        form4.setHash("hash2");
        formDao.save(form4);
        FormPair pair2 = new FormPair();
        pair2.setState("new");
        pair2.setManagersForm(form3);
        pair2.setStudentsForm(form4);
        formPairDao.save(pair2);
        pairs = formPairDao.getAllFormPairs();
        assertEquals(2, pairs.size());
        assertTrue(pairs.contains(pair));
        assertTrue(pairs.contains(pair2));
        pairs = formPairDao.getAllFormPairs(period);
        assertEquals(1, pairs.size());
        assertTrue(pairs.contains(pair));
        formPairDao.delete(pair);
        formPairDao.delete(pair2);
        periodDao.delete(period);
    }

}
