package cz.muni.fi.test;

import cz.muni.fi.dao.OrganizationDao;
import cz.muni.fi.dao.ParticipantDao;
import cz.muni.fi.domain.Manager;
import cz.muni.fi.domain.Organization;
import cz.muni.fi.domain.Participant;
import org.junit.After;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import java.util.List;

import static org.junit.Assert.*;

/**
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 * @version 20.8.2011
 */

@RunWith(SpringJUnit4ClassRunner.class)
// ApplicationContext will be loaded from "/applicationContext.xml" and "/applicationContext-test.xml"
// in the root of the classpath
@ContextConfiguration(locations = {"/applicationContext-test.xml"})
public class JpaParticipantDaoTest {

    @Autowired
    private ParticipantDao participantDao;

    @Autowired
    private OrganizationDao organizationDao;

    @Test
    public void testFindById() throws Exception {
        Participant participant = createSampleStudent();
        participantDao.save(participant);
        Participant result = participantDao.findByEmail(participant.getMail());
        assertNotNull(result);
        assertEquals(participant, result);
        assertEquals(participant.getMail(), result.getMail());
        assertEquals(participant.getName(), result.getName());
        assertEquals(participant.getRole(), result.getRole());

        result = participantDao.findByEmail("id");
        assertNull(result);

        participantDao.delete(participant);
    }

    @After
    public void tearDown() {
        List<Organization> list = organizationDao.getAllOrganizations();
        assertTrue(list.isEmpty());
    }

    @Test
    public void testSave() throws Exception {
        Manager manager = createSampleManager();
        assertNull(manager.getOrganization().getId());
        organizationDao.save(manager.getOrganization());
        assertNotNull(manager.getOrganization().getId());
        participantDao.save(manager);
        Participant result = participantDao.findByEmail(manager.getMail());
        assertNotNull(result);
        assertEquals(manager, result);
        assertEquals(manager.getMail(), result.getMail());
        assertEquals(manager.getName(), result.getName());
        assertEquals(manager.getRole(), result.getRole());
        assertTrue(result instanceof Manager);
        assertNotNull(((Manager) result).getOrganization());
        assertNotNull(((Manager) result).getOrganization().getId());
        assertEquals(manager.getOrganization(), ((Manager) result).getOrganization());
        assertEquals(manager.getOrganization().getName(), ((Manager) result).getOrganization().getName());
        Organization org1 = organizationDao.findById(manager.getOrganization().getId());
        assertNotNull(org1);
        assertEquals(manager.getOrganization(), org1);
        Manager manager2 = createSampleManager();
        manager2.setMail("man2@org.com");
        manager2.setOrganization(manager.getOrganization());
        assertNotNull(manager2.getOrganization().getId());
        participantDao.save(manager2);
        result = participantDao.findByEmail(manager2.getMail());
        assertNotNull(result);
        assertEquals(manager2, result);
        assertEquals(manager2.getMail(), result.getMail());
        assertEquals(manager2.getName(), result.getName());
        assertEquals(manager2.getRole(), result.getRole());
        assertTrue(result instanceof Manager);
        assertNotNull(((Manager) result).getOrganization());
        List<Participant> mans = participantDao.getAllManagers();
        assertEquals(2, mans.size());
        List<Organization> orgs = organizationDao.getAllActiveOrganizations();
        assertEquals(1, orgs.size());
        participantDao.delete(manager);
        participantDao.delete(manager2);
        organizationDao.delete(org1);

    }

    private Manager createSampleManager() {
        Manager manager = new Manager();
        Organization org = new Organization();
        org.setName("org");
        org.setActive(true);
        manager.setMail("managerQ@org.com");
        manager.setName("manager");
        manager.setRole("manager");
        manager.setOrganization(org);
        return manager;
    }

    @Test
    public void testUpdate() throws Exception {
        Manager manager = createSampleManager();
        organizationDao.save(manager.getOrganization());
        participantDao.save(manager);
        Participant result = participantDao.findByEmail(manager.getMail());
        assertNotNull(result);
        assertEquals(manager, result);
        assertEquals(manager.getMail(), result.getMail());
        assertEquals(manager.getName(), result.getName());
        assertEquals(manager.getRole(), result.getRole());
        assertTrue(result instanceof Manager);
        assertNotNull(((Manager) result).getOrganization());
        assertEquals(manager.getOrganization(), ((Manager) result).getOrganization());
        assertEquals(manager.getOrganization().getName(), ((Manager) result).getOrganization().getName());

        manager.setName("koukal");
        participantDao.update(manager);
        result = participantDao.findByEmail(manager.getMail());
        assertNotNull(result);
        assertEquals(manager, result);
        assertEquals(manager.getMail(), result.getMail());
        assertEquals(manager.getName(), result.getName());
        assertEquals(manager.getRole(), result.getRole());
        assertTrue(result instanceof Manager);
        assertNotNull(((Manager) result).getOrganization());
        assertEquals(manager.getOrganization(), ((Manager) result).getOrganization());
        assertEquals(manager.getOrganization().getName(), ((Manager) result).getOrganization().getName());

        Organization org1 = manager.getOrganization();

        Organization org2 = new Organization();
        org2.setName("org2");
        organizationDao.save(org2);
        manager.setOrganization(org2);
        participantDao.update(manager);
        result = participantDao.findByEmail(manager.getMail());
        assertNotNull(result);
        assertEquals(manager, result);
        assertEquals(manager.getMail(), result.getMail());
        assertEquals(manager.getName(), result.getName());
        assertEquals(manager.getRole(), result.getRole());
        assertTrue(result instanceof Manager);
        assertNotNull(((Manager) result).getOrganization());
        assertEquals(manager.getOrganization(), ((Manager) result).getOrganization());
        assertEquals(manager.getOrganization().getName(), ((Manager) result).getOrganization().getName());
        assertEquals(org2, ((Manager) result).getOrganization());

        participantDao.delete(manager);
        organizationDao.delete(manager.getOrganization());
        organizationDao.delete(org1);

    }

    @Test
    public void testDelete() throws Exception {
        Manager manager = createSampleManager();
        manager.setMail("mandel@org.com");
        organizationDao.save(manager.getOrganization());
        participantDao.save(manager);
        Participant result = participantDao.findByEmail(manager.getMail());
        assertNotNull(result);
        participantDao.delete(manager);
        result = participantDao.findByEmail(manager.getMail());
        assertNull(result);
        Organization org = organizationDao.findById(manager.getOrganization().getId());
        assertNotNull(org);
        assertEquals(manager.getOrganization().getName(), org.getName());
        organizationDao.delete(org);


    }

    @Test
    public void testGetAllStudents() throws Exception {
        Manager manager = createSampleManager();
        organizationDao.save(manager.getOrganization());
        participantDao.save(manager);
        List<Participant> studs = participantDao.getAllStudents();
        assertEquals(0, studs.size());

        Participant student = createSampleStudent();
        participantDao.save(student);
        studs = participantDao.getAllStudents();
        assertEquals(1, studs.size());
        assertTrue(studs.contains(student));

        Participant student2 = createSampleStudent();
        student2.setMail("student2@org.com");
        participantDao.save(student2);

        studs = participantDao.getAllStudents();
        assertEquals(2, studs.size());
        assertTrue(studs.contains(student));
        assertTrue(studs.contains(student2));

        participantDao.delete(manager);
        organizationDao.delete(manager.getOrganization());
        participantDao.delete(student);
        participantDao.delete(student2);

    }

    @Test
    public void testGetAllManagers() throws Exception {
        List<Participant> mans = participantDao.getAllManagers();
        assertEquals(0, mans.size());

        Manager manager = createSampleManager();
        manager.setMail("mandel@org.com");
        organizationDao.save(manager.getOrganization());
        participantDao.save(manager);
        mans = participantDao.getAllManagers();
        assertEquals(1, mans.size());
        assertTrue(mans.contains(manager));

        Manager man2 = createSampleManager();
        man2.setOrganization(manager.getOrganization());
        participantDao.save(man2);
        mans = participantDao.getAllManagers();
        assertEquals(2, mans.size());
        assertTrue(mans.contains(manager));
        assertTrue(mans.contains(man2));

        Participant student = createSampleStudent();
        participantDao.save(student);
        mans = participantDao.getAllManagers();
        assertEquals(2, mans.size());
        assertTrue(mans.contains(manager));
        assertTrue(mans.contains(man2));

        participantDao.delete(man2);
        participantDao.delete(manager);
        organizationDao.delete(manager.getOrganization());
        participantDao.delete(student);
    }


    @Test
    public void testCanBeDeleted() {
        Participant partip = createSampleStudent();
        participantDao.save(partip);
        assertTrue(participantDao.canBeDeleted(partip));
        participantDao.delete(partip);
    }

    private Participant createSampleStudent() {
        Participant student = new Participant();
        student.setMail("studentQ@org.com");
        student.setRole("student");
        student.setName("student");
        return student;

    }

}
